// SIHASTIDENT01.inc
// Description: Identification Check for HART
// Copyright (C) Siemens AG (1999-2003). All Rights Reserved.
// $Revision: 1.1 $
// $Date: 2010/01/25 14:13:01EST $

#define USING_LONG_TAG

#include "Manufacturer_List.h"

#define ID_CHECK_ABORT       0
#define ID_CHECK_SUCCESS     1

#define ID_STATUS_SIZE        3     /* size of status array            */
#define ID_STATUS_COMM_STATUS 1

#define ID_FC_ERROR          3000

#define COMMAND_ID_READ_UNIQUE_IDENTIFIER      0
#define COMMAND_ID_READ_PV_OUTPUT_INFO         15
#define COMMAND_ID_READ_LONG_TAG               20

#define LONGTAG_MISMATCH \
	"The Long TAG is not as expected !\nPossible cause:\n* The Long TAG has been changed by someone else \n* You did not save to disk the last time you modified the Long TAG" \
	"|de|Die TAG Lang (Gertekennzeichen) ist nicht wie erwartet !\nMgliche Ursache:\n* Die TAG Lang wurde von jemanden gendert \n* Sie haben das letzte mal nicht gespeichert als Sie die TAG Lang gendert haben" \
	"|fr|Le TAG Longue de l'appareil n'est pas celui attendu !\nCause possible:\n* Le TAG Longue de l'appareil a t modifi par un autre utilisateur\n* Vous n'avez pas sauvegard sur le disque lors de votre dernire modification du TAG Longue de l'appareil." \
	"|sp|El TAG Larga (identificacin del aparato) no es el previsto!\nCausa posible:\n* El TAG Larga ha sido modificado \n* No se memoriz el TAG Larga despus de modificarlo."

#define LONGTAG_MISMATCH_3 \
	"The Long TAG of the device is different from the Long TAG in the offline data !\n\nOffline data: %{longTag}\n\nDevice: %{id_longTag}\n" \
	"|de|Die TAG Lang des Gertes ist ungleich der TAG Lang in der Projektdatenhaltung !\n\nProjektdatenhaltung: %{longTag}\n\nGert: %{id_longTag}\n" \
   "|fr|Le TAG Longue de l'appareil diffre du TAG Longue de la base de donnes du projet !\n\nBase de donnes du projet : %{longTag}\n\nAppareil : %{id_longTag}" \
	"|sp|El TAG Larga del aparato no corresponde al TAG Larga de la base de datos del proyecto!\n\nBase de datos del proyecto: %{longTag}\n\nAparato: %{id_longTag}"

#define LONGTAG_MISMATCH_APPLY \
	"Use Long TAG from device" \
	"|de|TAG Lang des Gerts verwenden" \
	"|fr|Utiliser le TAG Longue de l'appareil" \
	"|sp|Utilizar el TAG Larga del aparato"

#define LONGTAG_MISMATCH_KEEP_OLD \
	"Use Long TAG from offline data" \
	"|de|TAG Lang aus der Projektdatenhaltung verwenden" \
	"|fr|Utiliser le TAG Longue de l'appareil de la base de donnes du projet" \
	"|sp|Utilizar el TAG Larga de la base de datos del proyecto"

// --- Identcheck and WriteProtectionCheck use features only supported in 5.0.2 !

// enable ident check
#define ENABLE_IDENTCHECK

// enable WriteProtection check
#define ENABLE_WRITEPROTECTIONCHECK

// Additional ITEMS for Options Menu 
// #define DEBUG_OPTION1   OfflineDialog_DebugDeviceIdentification

// --- IdentCheck DEBUG Dialog ------------------------------------------

#ifdef DEBUG_OPTION1

MENU OfflineDialog_DebugDeviceIdentification
{
   LABEL "Test_Identification";
   ITEMS
   {
      Page_DebugDeviceIdentification,
      Page_Id_ConfigChangedFlag
   }
}

MENU Page_DebugDeviceIdentification
{
   LABEL "Test: Identification";
   ITEMS
   {
      group_identcheck_dd,
      SEPARATOR,
      group_identcheck_device,
   }
}

MENU group_identcheck_dd
{
   LABEL "Offline data" ;
   ITEMS
   {
      longTag (READ_ONLY),
      dd_manufacturer_id,
      dd_device_type,
      device_id,
      dd_device_revision
   }
}

MENU group_identcheck_device
{
   LABEL "Device" ;
   ITEMS
   {
      id_longTag,
      id_manufacturer_id,
      id_device_type,
      id_device_id,
      id_device_revision,

      std_CheckDeviceIdentification,
      std_CheckDeviceIdentificationUpload,
      std_CheckDeviceIdentificationDownload,

      ret_CheckDeviceIdentification
   }
}

MENU Page_Id_ConfigChangedFlag
{
   LABEL "Test: Config Changed Flag";
   ITEMS
   {
      ret_ConfigChangedState (READ_ONLY),
      device_status(READ_ONLY)
      std_GetConfigChangedState,
      std_ResetConfigChangedState
   }
}

#endif

// --- IdentCheck VARIABLES ------------------------------------------

VARIABLE ret_CheckDeviceIdentification
{
   LABEL    "Result";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (1)
   {
      DEFAULT_VALUE  ID_CHECK_SUCCESS ;
   }
   HANDLING READ;
}

VARIABLE ret_CheckTAGIdentity
{
   LABEL    "ret_CheckTAGIdentity";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (1)
   {
      DEFAULT_VALUE  1;
   }
   HANDLING READ;
}

VARIABLE id_Method_In_SilentMode
{
    LABEL      "Method in Silent Mode";
    CLASS      LOCAL;
    TYPE    ENUMERATED
    {
       DEFAULT_VALUE 0x00; 
       { 0x00, [no]  },
        { 0x01, [yes]   }
   }
    HANDLING   READ;
}

VARIABLE id_firstCheck
{
   LABEL    "id_firstCheck";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (1)
   {
      DEFAULT_VALUE  1;
   }
   HANDLING READ;
}

VARIABLE id_status
{
   LABEL    "";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (1)
   {
      DEFAULT_VALUE  ID_CHECK_ABORT;
   }
   HANDLING READ;
}

VARIABLE id_dummy_dword
{
   LABEL    "";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (4);
   HANDLING READ;
}

VARIABLE id_dummy_word
{
   LABEL    "";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (2);
   HANDLING READ;
}

VARIABLE id_dummy_byte
{
   LABEL    "";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (1);
   HANDLING READ;
}

VARIABLE id_dummy_float
{
   LABEL    "";
   CLASS    LOCAL;
   TYPE     FLOAT ;
   HANDLING READ;
}

VARIABLE id_manufacturer_id
{
   HELP [manufacturer_id_code_help];
   LABEL [manufacturer_id];
   HANDLING READ;
   CLASS LOCAL ;
   TYPE ENUMERATED
    {
     DEFAULT_VALUE 252 ;

      MANUFACTURER_LIST
    }
}

VARIABLE id_device_type
{
   LABEL    [device_type];
   HELP     [manufacturers_device_type_help];
   HANDLING READ;
   CLASS    LOCAL ;
   TYPE     ENUMERATED (1)
   {
      SELECT( id_manufacturer_id )
      { 
      CASE 10:  // BROOKS INSTRUMENT
         { 1, [tri20],          "Flow          Totalizer - Rate Indicator"  },
         { 2, [brooks38xxva],   "Flow          Variable Area" },
         { 3, [brooks99xxoval], "" },
         { 4, [quantim],        "Flow          Coriolis Mass" }

      CASE 14:  // DELTA
         {1, "HT"}

      CASE 17:  // ENDRESS HAUSER
        { 3,   [fmu860],     "Level         Ultrasonic" },
        { 4,   [fmu861],     "Level         Ultrasonic" },
        { 5,   [fmu862],     "Level         Ultrasonic" },
        { 6,   [fmr130],     "Level         Radar - Microwave " },
        { 7,   [cerabar_s],  "Pressure      Absolute" },
        { 8,   [fec12],      "Level         Capacitance" },
        { 9,   [deltabar_s], "Pressure      Differential" },
        { 10,  [fmu231],     "Level         Ultrasonic" },
        { 11,  [deltapilot], "Level         Hydrostatic" },
        { 12,  [fmr23x],     "Level         Ultrasonic" },
        { 13,  [fmp200],     "Level" },
        { 14,  [cerabar_m],  "Pressure      Absolute/Relative" },
        { 15,  [fmr2xx],     "Level         Microwave" },
        { 16,  [fmr53x],     "Level         Microwave" },
        { 17,  [fmu4x],      "Level         Ultrasonic" },
        { 18,  [fmp4x],      "Level         Radar" },
        { 19,  [fmg60],      "Level         Density - Nuclear" },
        { 20,  [nrf590] },
        { 23,  [deltabars],  "Pressure      Deltabar S Evolution" },
        { 24,  [cerabars],   "Pressure      Cerabar S Evolution" },
        { 30,  [fmr25x],     "Level         Microwave" },
        { 50,  [promag33],   "Flow          Electromagnetic" },
        { 51,  [prowirl70],  "Flow          Vortex" },
        { 52,  [promass63],  "Flow          Coriolis Mass" },
        { 53,  [promag39],   "Flow          Magmeter - Rack Version"},
        { 54,  [promag35s],  "Flow          High Performance Magmeter" },
        { 55,  [prowirl77],  "Flow          Vortex" },
        { 57,  [promass60],  "Flow          Mass" },
        { 64,  [proson_f],   "Flow          Ultrasonic" },
        { 65,  [promag50],   "Flow          Magnetic" },
        { 66,  [promag53],   "Flow          Magnetic" },
        { 67,  [promag51],   "Flow          Magnetic - Custody Transfer" },
        { 69,  [promag10],   "Flow          Magnetic" },
        { 70,  [promag23],   "Flow          2-wire Magnetic" },
        { 80,  [promass80],  "Flow          Coriolis Mass" },
        { 81,  [promass83],  "Flow          Coriolis Mass" },
        { 82,  [promass84],  "Flow          Coriolis Mass - Custody Transfer" },
        { 83,  [promass40],  "Flow          Coriolis Mass" },
        { 86,  [prowirl72],  "Flow          2-wire Vortex" },
        { 87,  [prowirl73],  "Flow          2-wire Vortex" }
        { 88,  [pflow90],    "Flow          Ultrasonic" },
        { 89,  [pflow93],    "Flow          Ultrasonic" }
        { 100, [at70],       "Flow          Thermal" },
        { 120, [tmd832],     "Temperature   Head Mounted" },
        { 121, [tmd833],     "Temperature   Compact" },
        { 124, [tmd842],     "Temperature   DIN Rail Mounted" },
        { 140, [mypro_ph],   "Analytical    pH" },
        { 141, [mypro_lfc],  "Analytical    Conductivity - Conductive" },
        { 142, [mypro_lfi],  "Analytical    Conductivity - Inductive" },
        { 181, [nmt530],     "Temperature   Sensor" },
        { 182, [nms530],     "Level         Servo" },
        { 200, [tmt182],     "Temperature   Head Mounted" },
        { 201, [tmt122],     "Temperature   DIN Rail" },
        { 202, [tmt162],     "Temperature" },
        { 203, [tmt142],     "Temperature" }

      CASE 18:  // BAILEY FISCHER AND PORTER
         {7, "50XM2000"},
         {8, "50XE4000"},
         {14, "50VT1000"},
         {15, "50VM1000"},
         {25, "50XM1000"},
         {26, "50SM1000"},
         {66, "PTH"}

      CASE 19:  // FISHER CONTROLS
         {1, "DT"},
         {2, "DVC"}

      CASE 20:  // FOXBORO
         { 41,  [imt25],       "Flow          Magnetic" },
         { 46,  [ia_pressure], "Pressure" },
         { 47,  [imv_25],      "Pressure      Multivariable" },
         { 48,  [fox_875ph],   "Analytical    pH/ORP" },
         { 51,  [itvortex],    "Flow          Vortex" },
         { 52,  [cft50],       "Flow          Coriolis" },
         { 53,  [fox_875cr],   "Analytical    Conductivity" },
         { 54,  [fox_875ec],   "Analytical    Conductivity" },
         { 56,  [rtt15],       "Temperature" },
         { 125, [cft30x],      "Flow          Coriolis" }

      CASE 21:  // FUJI
        { 1,   [fcx_ac],   "Pressure"},
        { 2,   [fcx_a2c2], "Pressure" },
        { 129, [frc ],     "Temperature" }

      CASE 22:  // HARTMANN BRAUN
         { 3,   [teu471] },
         { 4,   [teu421] },
         { 5,   [teu211], "Temperature" },
         { 6,   [ts11],   "Temperature" },
         { 8,   [th02],   "Temperature" },
         { 64,  [tzid],   "Valve         Positioner/Pneumatic" },
         { 65,  [tzidc],  "Valve         Positioner/Pneumatic" },
         { 133, [as800],  "Pressure" }
         { 135, [contrac],"Valve         Actuator - Electrical" }

      CASE 23:  // HONEYWELL
         { 1, [st3000],    "Pressure" },
         { 2, [stt25t],    "Temperature   Dual Input" },
         { 3, [hwflow],    "Flow          Magnetic" },
         { 4, [stt25h],    "Temperature" },
         { 5, [herculine], "Valve         Actuator" },
         { 6, [smartcet],  "Analytical    Corrosion" },
         { 7, [stt17h],    "Temperature" },
         { 8, [stt2500] }

      CASE 25:  // KAY RAY SENSALL
         {8, "4050"},
         {9, "3680"},
         {10, "4790"},
         {15, "3280"}

      CASE 26:  // ABB
         { 1,   [ksx],               "Pressure" },
         { 2,   [abb_600t],          "Pressure      Differential" },
         { 3,   [abb_2600t_268],     "Pressure      Safety" },
         { 4,   [abb_2600t_262_264], "Pressure" },
         { 8,   [abb_kst],           "Temperature" },
         { 10,  [abb_658t],          "Temperature" },
         { 12,  [abb_652],           "Temperature" },
         { 29,  [fsm4000],           "Flow          Electromagnetic" },
         { 35,  [tb82ph_comb],       "Analytical    pH" },
         { 36,  [tb82ec_comb],       "Analytical    4 Electrode Conductivity" },
         { 37,  [tb82te_comb],       "Analytical    2 Electrode Conductivity" },
         { 38,  [tb82tc_comb],       "Analytical    Troidal Conductivity" },
         { 137, [abb_263_265_2000t], "Pressure      P, dP + Multivariable" },
         { 140, [abb_261],           "Pressure      Absolute" }

      CASE 31:  // MICRO MOTION
         { 7,  [mass_flow_9712], "Flow          Coriolis Mass" },
         { 20, [mass_flow_9720] },
         { 21, [mass_flow_9739], "Flow          Coriolis Mass" },
         { 30, [mass_flow_9701], "Flow          Coriolis Mass" },
         { 36, [mvd_2000_io],    "Flow          Coriolis Mass" },
         { 37, [mvd_1000_io],    "Flow          Coriolis Mass" },
         { 38, [mvd_2000_is],    "Flow          Coriolis Mass" },
         { 39, [mvd_1000_is],    "Flow          Coriolis Mass" },
         { 41, [mvd_1000_ao],    "Flow          Coriolis Mass" },
         { 42, [mvd_2000_ao],    "Flow          Coriolis Mass" },
         { 52, [mass_flow_2400s],"Flow          Coriolis Mass" },
         { 65, [mvd_3000],       "Flow          Coriolis Mass" }

      CASE 32:  // MOORE INDUSTRIES
        { 1, [trz], "Temperature" },
        { 3, [thz], "Temperature" }

      CASE 33:  // MOORE PRODUCTS
         { 1,  [moore340a] },
         { 2,  [moore340b] },
         { 3,  [moore344] },
         { 5,  [prime330],   "Pressure"},
         { 6,  [moore340a2], "Pressure      w/ PID" },
         { 8,  [moore340b2] },
         { 9,  [moore3442] },
         { 10, [moore340s],  "Pressure      Saturated Steam" },
         { 11, [moore343],   "Temperature" },
         { 12, [moore760d] },
         { 13, [moore345],   "Pressure"}

      CASE 38:  // ROSEMOUNT
         { 1,   [pressure_3051] },
         { 2,   [temperature_3044],      "Temperature" },
         { 3,   [pressure_1151s],        "Pressure" },
         { 4,   [mag_flow_8712],         "Flow          Magnetic" },
         { 5,   [pressure_2001] },
         { 6,   [pressure_3051c],        "Pressure" },
         { 11,  [pressure_3001l] },
         { 12,  [mag_flow_8712H],        "Flow          Magnetic"  },
         { 13,  [temperature_3044C],     "Temperature" },
         { 14,  [pressure_3001c],        "Pressure" },
         { 15,  [pressure_3051clp],      "Pressure" },
         { 16,  [vortex_flow_8800],      "Flow          Vortex" },
         { 17,  [htg_3201],              "Level         Hydrostatic" },
         { 18,  [pressure_1152s] },
         { 19,  [htg_3202],              "Level         Hydrostatic" },
         { 20,  [pressure_3001s],        "Pressure" },
         { 21,  [multi_3095FT] },
         { 22,  [mv_3095],               "Flow          Multivariable Mass" },
         { 23,  [temperature_544] },
         { 24,  [temperature_644],       "Temperature" },
         { 25,  [temperature_3144],      "Temperature" },
         { 26,  [temperature_3244],      "Temperature   Multivariable" },
         { 27,  [level_3700] },
         { 28,  [level_3750],            "Level" },
         { 29,  [TRI_LOOP],              "Specialty     HART-to-Analog Signal Conv" },
         { 30,  [c_3095],                "Level" },
         { 31,  [mm_3095] },
         { 32,  [tank_3701] },
         { 33,  [level_3300],            "Level         Radar" },
         { 34,  [asu_3702] },
         { 35,  [pressure_2088s],        "Pressure" },
         { 36,  [probar],                "Flow          dP" },
         { 37,  [massprobar] },
         { 38,  [prov] },
         { 39,  [pressure_2090s],        "Pressure" },
         { 40,  [probar_uc],             "Flow          dP" }, /* Probar UC */
         { 41,  [d_2055] }, /* 2055D */
         { 42,  [pv_3095] }, /* 3095PV */
         { 45,  [sentry] }  /* Sentry */
         { 47,  [proplate_uc],           "Flow          dP" },
         { 54,  [pressure_1810],         "Pressure" },
         { 55,  [mag_flow_8712d],        "Flow          Magnetic" },
         { 56,  [pressure_4600],         "Pressure" },
         { 59,  [temperature_248],       "Temperature" },
         { 60,  [mag_flow_8732d],        "Flow          Magnetic" },
         { 64,  [pressure_2088lp],       "Pressure      Low Power" },
         { 65,  [level_5600] },
         { 66,  [presssure_951],         "Pressure" },
         { 67,  [level_5400],            "Level         Radar" },
         { 68,  [pressure_3051s_sis],    "Pressure      Certified SIL2" },
         { 69,  [temperature_3144p_sis], "Temperature" },
         { 71,  [pressure_4500],         "Pressure" },
         { 128, [generic],               "Interoperable" }

      CASE 39:  // PEEK MEASUREMENT
         {0, "FD900"}

      CASE 40:  // SCHLUMBERGER
         {1, "NEXGEN"}

      CASE 42:  // SIEMENS
         { 1,  [micro_k] },
         { 2,  [sitrans_l] },
         { 3,  [sipan_ph] },
         { 4,  [sitrans_fus],        "Flow          Ultrasonic" },
         { 5,  [sipan_lf] },
         { 6,  [sipan_o2] },
         { 7,  [sitrans_lr],         "Level         Radar" },
         { 10, [sitrans_phs],        "Pressure" },
         { 11, [sitrans_pds],        "Pressure" },
         { 12, [sitrans_pes],        "Pressure" },
         { 13, [sitrans_pms],        "Pressure" },
         { 15, [sitrans_t2k] },
         { 16, [sitrans_tw],         "Temperature" },
         { 18, [sitrans_tkh],        "Temperature" },
         { 19, [sitrans_th300],      "Temperature" },
         { 21, [sipart_ps2],         "Valve         Positioner/Electropneumatic" },
         { 22, [sitrans_fm],         "Flow          Electromagnetic" },
         { 23, [sitrans_fva] },
         { 24, [mag6000],            "Flow          Magnetic Inductive" },
         { 26, [sitrans_fm_it2_tm2], "Flow" },
         { 29, [sitrans_p300],       "Pressure" }

      CASE 44:  // TOSHIBA
         { 2,  [ap3100], "Pressure" },
         { 3,  [ap3110] },
         { 4,  [ap3120] },
         { 5,  [ap3140] },
         { 6,  [ap3150] },
         { 7,  [ap3170] },
         { 8,  [ap3180] },
         { 9,  [ap3190] },
         { 10, [lf220], "Flow          Magnetic" }

      CASE 46:  // ROSEMOUNT ANALYTICAL
         { 6,  [ph_1181],              "Analytical    pH" },
         { 7,  [conductivity_1181],    "Analytical    Conductivity" },
         { 10, [ph_1054] },
         { 11, [conductivity_1054] },
         { 12, [oxymitter_4000],       "Analytical    Oxygen" },
         { 13, [wc_3000],              "Analytical" },
         { 14, [ra3081fg],             "Analytical" },
         { 15, [opm_2000r],            "Analytical    Opacity Monitor" },
         { 16, [ocx_4000],             "Analytical    Oxygen/Combustion" },
         { 20, [ph_3081],              "Analytical    pH" },
         { 21, [c_3081],               "Analytical    2-wire Conductivity" },
         { 22, [ra3081_81con],         "Analytical    2-wire Conductivity w/ pure H20" },
         { 23, [ra3081_81t],           "Analytical    2-wire Conductivity w/ % conc" },
         { 24, [ocx_cl_oz_5081a],      "Analytical    2-wire Oxygen/Chlorine/Ozone" },
         { 25, [conductivity_5081c_t], "Analytical    2-wire Conductivity" },
         { 26, [ph_5081],              "Analytical    2-wire pH/ORP" },
         { 27, [xmt_a],                "Analytical    2-wire Oxygen/Chlorine/Ozone" },
         { 28, [xmt_c_t],              "Analytical    2-wire Contacting/Toroidal" },
         { 29, [xmt_ph],               "Analytical    2-wire pH/ORP" },
         { 30, [oxygen_5081fg],        "Analytical    Oxygen - High Temperature" },
         { 80, [ph_orp_54],            "Analytical    pH/ORP" },
         { 81, [c_54],                 "Analytical    Conductivity w/ % conc" }
         { 82, [ra52eph_orp],          "Analytical    pH/ORP/ISE" },
         { 83, [ra54ea],               "Analytical    Amperometric Analyzer" }

      CASE 47:  // VALMET
         { 1,  [psmart],    "Pressure      Pressure/Level" },
         { 60, [smartpulp], "Analytical    Consistency" },
         { 61, [smartlx],   "Analytical    Consistency" },
         { 62, [mcai],      "Analytical    Microwave-Pulp Consistency" },
         { 63, [smartlc],   "Analytical    Consistency" }

      CASE 48:  // FLOWSERVE
         { 1, [logix_12xx],   "Valve         Digital Positioner" },
         { 2, [kemmer_c2100], "Valve         Positioner" },
         { 3, [logix_520],    "Valve         Digital Positioner" },
         { 4, [Logix_3200iq], "Valve         Digital Positioner" }

      CASE 50:  // VIATRAN
         { 1,  [viatran970],          "Pressure" },
         { 46, [viatran_ia_pressure], "Pressure" }

      CASE 54:  // YAMATAKE HONEYWELL
         { 1, [magnew],     "Flow          Magnetic" },
         { 2, [st3000],     "Pressure" },
         { 3, [svp],        "Valve         Smart Valve Positioner" },
         { 4, [thermoplus], "Temperature" },
         { 5, [ptg],        "Pressure" },
         { 8, [magnew_2w],  "Flow          Electromagnetic" }

      CASE 55:  // YOKOGAWA
         { 1,  [yewflo],   "Flow          Vortex" },
         { 2,  [yt200],    "Temperature   Differential" },
         { 3,  [unicom],   "Pressure" },
         { 4,  [eja],      "Pressure      Differential" },
         { 5,  [admag_ae], "Flow          Magnetic" },
         { 6,  [am11],     "Flow" },
         { 8,  [admag_se], "Flow" },
         { 9,  [yta],      "Temperature" },
         { 10, [yta70e],   "Temperature" },
         { 11, [dyf],      "Flow          Vortex" },
         { 12, [zr202],    "Analytical    Oxygen" },
         { 13, [zr402],    "Analytical    Oxygen" },
         { 20, [isc202],   "Analytical    Conductivity - Inductive" },
         { 21, [ph202],    "Analytical    pH" },
         { 22, [sc202],    "Analytical    Conductivity" },
         { 24, [do202],    "Analytical    Dissolved Oxygen" },
         { 39, [ph150],    "Analytical    pH Controller" },
         { 40, [sc150],    "Analytical    Conductivity/Resistivity Controller" },
         { 43, [ph450],    "Analytical    pH" },
         { 44, [sc450],    "Analytical    Conductivity/Resistivity" },
         { 45, [isc450],   "Analytical    Conductivity - Inductive" },
         { 64, [rotamass], "Flow          Mass" },
         { 65, [ramc],     "Flow          Variable Area" },
         { 66, [rcct_f3],  "Flow          Mass"},
         { 80, [axfa11],   "Flow          Magnetic" }
         { 81, [ejx],      "Pressure" },
         { 82, [axfa14],   "Flow          Magnetic" },
         { 83, [av550g],   "Analytical    Oxygen" },
         { 84, [ejx_mv],   "Pressure      Multivariable" }

      CASE 58:  // EXAC
         {211, "8320"},
         {212, "SX5100"}

      CASE 59:  // KDG MOBREY
         { 19, [solartron4301], "Pressure      w/ PID" },
         { 20, [solartron3301] },
         { 21, [msp100],        "Level" },
         { 33, [level_3300],    "Level         Radar" },
         { 41, [mlt100],        "Level         Displacement" },
         { 42, [solartron4291] },
         { 43, [msm400] },
         { 44, [msl600] },
         { 45, [mcu900] },
         { 46, [msp900_400] },
         { 47, [mrl700],        "Level         Radar" },
         { 48, [mrl800] },
         { 49, [mrl850] }

      CASE 61:  // PRINCO
         {1, "L4610"}

      CASE 62:  // SMAR
         { 1, [ld301], "Pressure      w/ PID" },
         { 2, [tt301], "Temperature   w/ PID" },
         { 3, [fy301], "Valve         Positioner/Electropneumatic" },
         { 4, [ld291], "Pressure" },
         { 5, [tp301], "Valve         Positioner" },
         { 6, [dt301], "Analytical    Density" }

      CASE 63:  // ECKARDT
         { 1, [tsv175],   "Temperature" },
         { 2, [dmu130],   "Pressure" },
         { 3, [ti_rtt20], "Temperature" },
         { 4, [srd991],   "Valve         Positioner" },
         { 5, [dmu140],   "Pressure" },
         { 6, [srd960],   "Valve         Universal Positioner" }

      CASE 66:  // SAMSON
        { 239, [samson373x_3], "Valve         Positioner" },
        { 249, [samson3780],   "Valve         Controller" }

      CASE 67:  // SPARLING INSTRUMENTS
           { 238, [fm6xx], "Flow          Magnetic" }

      CASE 69:  // KROHNE
         { 227, [ifc300], "Flow          Electromagnetic" },
         { 228, [optiflex1300c],   "Level         Radar" },
         { 229, [optiwave7300c],   "Level         Radar" },
         { 230, [m8e],    "Flow          Variable Area" },
         { 231, [ufc030], "Flow          Ultrasonic" },
         { 232, [mfc05x], "Flow          Coriolis Mass" },
         { 233, [ifc040], "Flow          Electromagnetic" },
         { 234, [m10],    "Flow          Variable Area" },
         { 235, [ifc210] },
         { 236, [bm102],  "Level         Radar - Reflex" },
         { 237, [vfm31],  "Flow          Vortex" },
         { 238, [bm100],  "Level         Radar - Reflex" },
         { 239, [ifc020] },
         { 240, [vfm1091] },
         { 242, [eskii],  "Flow          Variable Area" },
         { 243, [ifc110], "Flow          Magnetic" },
         { 244, [ifc090], "Flow          Magnetic"  },
         { 245, [ufc500], "Flow          Ultrasonic" },
         { 246, [ifc010] },
         { 247, [mfc08x], "Flow          Coriolis Mass" },
         { 248, [ifc080], "Flow          Magnetic" },
         { 249, [bm70],   "Level         Radar" }

      CASE 77:  // WESTLOCK CONTROLS
         { 1, [icot],       "Valve         Digital Positioner w/ PID" },
         { 2, [smartcal],   "Valve         Intelligent Positioner" },
         { 3, [d_epic],     "Valve         Digital Positioner" },
         { 4, [d_epic_esd], "Valve         Digital Positioner"}

      CASE 78:  // DREXELBROOK
           { 2, [universal_iii], "Level" }

      CASE 79:  // SAAB TANK CONTROL
         { 237, [saab_5600],    "Level         Radar" },
         { 239, [tankradarpro], "Level         Radar" }

      CASE 82:  // DRAEGER
         { 235, [polytron7000], "Analytical    Gas Detector" }
         { 237, [polytron2_ir], "Analytical    Infrared Gas Detector" },
         { 247, [polytron2],    "Analytical    Electrochemical Gas Detector" }

      CASE 86:  // MAGNETROL
         { 229, [model_705_3x],    "Level" },
         { 230, [model_rx5],    "Level" },
         { 231, [model_704],    "Level" },
         { 232, [jupiter],      "Level         Magnostrictive" },
         { 233, [model_ta2],    "Flow          Mass" },
         { 234, [model_707],    "Level" },
         { 235, [model_705_2x], "Level" },
         { 236, [model_708],    "Level" },
         { 237, [model_805],    "Level" },
         { 238, [model_705],    "Level" },
         { 239, [smartez],      "Level" }

      CASE 87:  // NELES JAMESBURY
         {239, "ND800"},
         {238, "ND800 PT"}

      CASE 88:  // MILLTRONICS
         { 200, [probe_86x],     "Level         Ultrasonic" },
         { 201, [sitrans_lr300], "Level         Radar - Pulse" },
         { 202, [lr_200],        "Level         Radar" },
         { 204, [sitrans_lr400], "Level         Radar" },
         { 205, [probe_lu6m],    "Level         Ultrasonic"},
         { 206, [probe_lu12m],   "Level         Ultrasonic"},
         { 207, [probe_lr],      "Level         Radar"},
         { 211, "LR 460 M",      "Level         Radar"}
         
      CASE 90:  // ANDERSON
         {200, "ANDRSN1"}

      CASE 91:  // INOR
         {239, "MESO"}

      CASE 92:  // ROBERTSHAW
         {200, "EXCALIBUR 7000"}

      CASE 94:  // ACCUTECH
         {239, "AI1500"}

      CASE 96:  // KAMSTRUP
         {238, "FLEXBAR HRT"},
         {239, "FLEXTOP HRT"}

      CASE 97:  // KNICK
         { 232, [condi_2211], "Analytical    Conductivity" },
         { 233, [oxy_2211],   "Analytical    Dissolved Oxygen" },
         { 234, [cond_2211],  "Analytical    Conductivity" },
         { 235, [ph_2211],    "Analytical    pH" }

      CASE 101:  // MASONEILAN
         { 100, [hdlt], "Level" },
         { 200, [svi],  "Valve         Positioner" },
         { 201, [svi2], "Valve         Positioner" }

      CASE 102:  // BESTA
         {239, "CLS420"}

      CASE 103:  // OHMART
         {224, "XMTR"}

      CASE 104:  // HAROLD BECK AND SONS
         {1, "ESR D"}

      CASE 107:  // WIKA
         {239, "T32H"}

      CASE 108:  // BOPP AND REUTHER HEINRICHS
         { 235, [umc2], "Flow          Mass" },
         { 236, [es],   "Flow          Electronic" },
         { 237, [vtx],  "Flow          Vortex" },
         { 238, [dimf], "Analytical    Density" },
         { 239, [ust1], "Flow" }

      CASE 109:  // PR ELECTRONICS
         { 238, [pretrans_6335], "Temperature" },
         { 239, [pretop_5335],   "Temperature" }

      CASE 113:  // APPARATEBAU HUNDSBACH
         {238, "MT115"}

      CASE 114:  // DYNISCO
         {224, "IPX"}

      CASE 120:  // BURKERT
           { 239, [burkert8630] }

      CASE 124:  // GLI
         { 238, [m33_53p] },
         { 239, [m33_53c] }

      CASE 126:  // PMC
         {224, "SMT EL"}

      CASE 127:  // LABOM
        { 60,  [pascal_ci], "Pressure" },
        { 224, [ipx],       "Pressure      Multivariable" }

      CASE 129:  // TURBO
           { 127, [intermag_transmag], "Flow          Magnetic" }

      CASE 131:  // SMC
         {239, "F793 E701"}

      CASE 133:  // HUAKONG
         {127, "HK TT01"}

      CASE 153:   // METRAN
         { 124, [metran_300prm] },
         { 125, [metran_49] },
         { 126, [metran_280] },
         { 127, [metran_100] }
         
      CASE 206:   // ROOST
         { 128, [epp300] }

      CASE 239:  // HCF
         {235, "STD"},
         {236, "SAMPLE4"},
         {237, "SAMPLE3"},
         {238, "SAMPLE2"},
         {239, "SAMPLE1"}
      }

      { 252, [unknown] }
   }
}

id_device_type          LIKE VARIABLE  device_type
{
   REDEFINE CLASS    LOCAL;
   DELETE PURPOSE;
}

VARIABLE id_device_id
{
   HELP [field_device_identification_help];
   LABEL [device_identification];
   HANDLING READ;
   CLASS LOCAL ;
   TYPE UNSIGNED_INTEGER (3);
}

id_device_revision      LIKE VARIABLE  transmitter_revision
{
   REDEFINE CLASS    LOCAL;
   DELETE PURPOSE;
}

id_longTag LIKE VARIABLE longTag
{
   REDEFINE CLASS    LOCAL;
   DELETE PURPOSE;
}

id_software_revision LIKE VARIABLE software_revision
{
   REDEFINE CLASS    LOCAL;
   DELETE PURPOSE;
}

VARIABLE id_write_protect
{
  HELP [write_protect_mode_help];
/*
  APPHELP "Common uses for the write_protect variable are to limit user access"
     " for Field Devices installed in custody transfer applications."
     " HART_VAR:WRITE PROT CODE";
*/
  LABEL [write_protect];
  HANDLING READ;
  CLASS LOCAL;
  TYPE ENUMERATED
    {
      { 0,  [not_write_protected] },
      { 1,  [write_protected] },
      { 250,   [not_used] },
      { 251,   [none] },
      { 252,   [unknown] },
      { 253,   [special] }
    }
}

dd_device_revision LIKE VARIABLE transmitter_revision
{
   REDEFINE CLASS    LOCAL;
}

dd_manufacturer_id         LIKE VARIABLE  id_manufacturer_id
{
   REDEFINE CLASS    LOCAL;
   REDEFINE HANDLING READ & WRITE ;
}

VARIABLE dd_device_type
{
   LABEL    [device_type];
   HELP     [manufacturers_device_type_help];
   HANDLING READ & WRITE ;
   CLASS    LOCAL ;
   TYPE     ENUMERATED (1)
   {
      SELECT( dd_manufacturer_id )
      { 
      CASE 10:  // BROOKS INSTRUMENT
         { 1, [tri20],          "Flow          Totalizer - Rate Indicator"  },
         { 2, [brooks38xxva],   "Flow          Variable Area" },
         { 3, [brooks99xxoval], "" },
         { 4, [quantim],        "Flow          Coriolis Mass" }

      CASE 14:  // DELTA
         {1, "HT"}

      CASE 17:  // ENDRESS HAUSER
        { 3,   [fmu860],     "Level         Ultrasonic" },
        { 4,   [fmu861],     "Level         Ultrasonic" },
        { 5,   [fmu862],     "Level         Ultrasonic" },
        { 6,   [fmr130],     "Level         Radar - Microwave " },
        { 7,   [cerabar_s],  "Pressure      Absolute" },
        { 8,   [fec12],      "Level         Capacitance" },
        { 9,   [deltabar_s], "Pressure      Differential" },
        { 10,  [fmu231],     "Level         Ultrasonic" },
        { 11,  [deltapilot], "Level         Hydrostatic" },
        { 12,  [fmr23x],     "Level         Ultrasonic" },
        { 13,  [fmp200],     "Level" },
        { 14,  [cerabar_m],  "Pressure      Absolute/Relative" },
        { 15,  [fmr2xx],     "Level         Microwave" },
        { 16,  [fmr53x],     "Level         Microwave" },
        { 17,  [fmu4x],      "Level         Ultrasonic" },
        { 18,  [fmp4x],      "Level         Radar" },
        { 19,  [fmg60],      "Level         Density - Nuclear" },
        { 20,  [nrf590] },
        { 23,  [deltabars],  "Pressure      Deltabar S Evolution" },
        { 24,  [cerabars],   "Pressure      Cerabar S Evolution" },
        { 30,  [fmr25x],     "Level         Microwave" },
        { 50,  [promag33],   "Flow          Electromagnetic" },
        { 51,  [prowirl70],  "Flow          Vortex" },
        { 52,  [promass63],  "Flow          Coriolis Mass" },
        { 53,  [promag39],   "Flow          Magmeter - Rack Version"},
        { 54,  [promag35s],  "Flow          High Performance Magmeter" },
        { 55,  [prowirl77],  "Flow          Vortex" },
        { 57,  [promass60],  "Flow          Mass" },
        { 64,  [proson_f],   "Flow          Ultrasonic" },
        { 65,  [promag50],   "Flow          Magnetic" },
        { 66,  [promag53],   "Flow          Magnetic" },
        { 67,  [promag51],   "Flow          Magnetic - Custody Transfer" },
        { 69,  [promag10],   "Flow          Magnetic" },
        { 70,  [promag23],   "Flow          2-wire Magnetic" },
        { 80,  [promass80],  "Flow          Coriolis Mass" },
        { 81,  [promass83],  "Flow          Coriolis Mass" },
        { 82,  [promass84],  "Flow          Coriolis Mass - Custody Transfer" },
        { 83,  [promass40],  "Flow          Coriolis Mass" },
        { 86,  [prowirl72],  "Flow          2-wire Vortex" },
        { 87,  [prowirl73],  "Flow          2-wire Vortex" }
        { 88,  [pflow90],    "Flow          Ultrasonic" },
        { 89,  [pflow93],    "Flow          Ultrasonic" }
        { 100, [at70],       "Flow          Thermal" },
        { 120, [tmd832],     "Temperature   Head Mounted" },
        { 121, [tmd833],     "Temperature   Compact" },
        { 124, [tmd842],     "Temperature   DIN Rail Mounted" },
        { 140, [mypro_ph],   "Analytical    pH" },
        { 141, [mypro_lfc],  "Analytical    Conductivity - Conductive" },
        { 142, [mypro_lfi],  "Analytical    Conductivity - Inductive" },
        { 181, [nmt530],     "Temperature   Sensor" },
        { 182, [nms530],     "Level         Servo" },
        { 200, [tmt182],     "Temperature   Head Mounted" },
        { 201, [tmt122],     "Temperature   DIN Rail" },
        { 202, [tmt162],     "Temperature" },
        { 203, [tmt142],     "Temperature" }

      CASE 18:  // BAILEY FISCHER AND PORTER
         {7, "50XM2000"},
         {8, "50XE4000"},
         {14, "50VT1000"},
         {15, "50VM1000"},
         {25, "50XM1000"},
         {26, "50SM1000"},
         {66, "PTH"}

      CASE 19:  // FISHER CONTROLS
         {1, "DT"},
         {2, "DVC"}

      CASE 20:  // FOXBORO
         { 41,  [imt25],       "Flow          Magnetic" },
         { 46,  [ia_pressure], "Pressure" },
         { 47,  [imv_25],      "Pressure      Multivariable" },
         { 48,  [fox_875ph],   "Analytical    pH/ORP" },
         { 51,  [itvortex],    "Flow          Vortex" },
         { 52,  [cft50],       "Flow          Coriolis" },
         { 53,  [fox_875cr],   "Analytical    Conductivity" },
         { 54,  [fox_875ec],   "Analytical    Conductivity" },
         { 56,  [rtt15],       "Temperature" },
         { 125, [cft30x],      "Flow          Coriolis" }

      CASE 21:  // FUJI
        { 1,   [fcx_ac],   "Pressure"},
        { 2,   [fcx_a2c2], "Pressure" },
        { 129, [frc ],     "Temperature" }

      CASE 22:  // HARTMANN BRAUN
         { 3,   [teu471] },
         { 4,   [teu421] },
         { 5,   [teu211], "Temperature" },
         { 6,   [ts11],   "Temperature" },
         { 8,   [th02],   "Temperature" },
         { 64,  [tzid],   "Valve         Positioner/Pneumatic" },
         { 65,  [tzidc],  "Valve         Positioner/Pneumatic" },
         { 133, [as800],  "Pressure" }
         { 135, [contrac],"Valve         Actuator - Electrical" }

      CASE 23:  // HONEYWELL
         { 1, [st3000],    "Pressure" },
         { 2, [stt25t],    "Temperature   Dual Input" },
         { 3, [hwflow],    "Flow          Magnetic" },
         { 4, [stt25h],    "Temperature" },
         { 5, [herculine], "Valve         Actuator" },
         { 6, [smartcet],  "Analytical    Corrosion" },
         { 7, [stt17h],    "Temperature" },
         { 8, [stt2500] }

      CASE 25:  // KAY RAY SENSALL
         {8, "4050"},
         {9, "3680"},
         {10, "4790"},
         {15, "3280"}

      CASE 26:  // ABB
         { 1,   [ksx],               "Pressure" },
         { 2,   [abb_600t],          "Pressure      Differential" },
         { 3,   [abb_2600t_268],     "Pressure      Safety" },
         { 4,   [abb_2600t_262_264], "Pressure" },
         { 8,   [abb_kst],           "Temperature" },
         { 10,  [abb_658t],          "Temperature" },
         { 12,  [abb_652],           "Temperature" },
         { 29,  [fsm4000],           "Flow          Electromagnetic" },
         { 35,  [tb82ph_comb],       "Analytical    pH" },
         { 36,  [tb82ec_comb],       "Analytical    4 Electrode Conductivity" },
         { 37,  [tb82te_comb],       "Analytical    2 Electrode Conductivity" },
         { 38,  [tb82tc_comb],       "Analytical    Troidal Conductivity" },
         { 137, [abb_263_265_2000t], "Pressure      P, dP + Multivariable" },
         { 140, [abb_261],           "Pressure      Absolute" }

      CASE 31:  // MICRO MOTION
         { 7,  [mass_flow_9712], "Flow          Coriolis Mass" },
         { 20, [mass_flow_9720] },
         { 21, [mass_flow_9739], "Flow          Coriolis Mass" },
         { 30, [mass_flow_9701], "Flow          Coriolis Mass" },
         { 36, [mvd_2000_io],    "Flow          Coriolis Mass" },
         { 37, [mvd_1000_io],    "Flow          Coriolis Mass" },
         { 38, [mvd_2000_is],    "Flow          Coriolis Mass" },
         { 39, [mvd_1000_is],    "Flow          Coriolis Mass" },
         { 41, [mvd_1000_ao],    "Flow          Coriolis Mass" },
         { 42, [mvd_2000_ao],    "Flow          Coriolis Mass" },
         { 52, [mass_flow_2400s],"Flow          Coriolis Mass" },
         { 65, [mvd_3000],       "Flow          Coriolis Mass" }

      CASE 32:  // MOORE INDUSTRIES
        { 1, [trz], "Temperature" },
        { 3, [thz], "Temperature" }

      CASE 33:  // MOORE PRODUCTS
         { 1,  [moore340a] },
         { 2,  [moore340b] },
         { 3,  [moore344] },
         { 5,  [prime330],   "Pressure"},
         { 6,  [moore340a2], "Pressure      w/ PID" },
         { 8,  [moore340b2] },
         { 9,  [moore3442] },
         { 10, [moore340s],  "Pressure      Saturated Steam" },
         { 11, [moore343],   "Temperature" },
         { 12, [moore760d] },
         { 13, [moore345],   "Pressure"}

      CASE 38:  // ROSEMOUNT
         { 1,   [pressure_3051] },
         { 2,   [temperature_3044],      "Temperature" },
         { 3,   [pressure_1151s],        "Pressure" },
         { 4,   [mag_flow_8712],         "Flow          Magnetic" },
         { 5,   [pressure_2001] },
         { 6,   [pressure_3051c],        "Pressure" },
         { 11,  [pressure_3001l] },
         { 12,  [mag_flow_8712H],        "Flow          Magnetic"  },
         { 13,  [temperature_3044C],     "Temperature" },
         { 14,  [pressure_3001c],        "Pressure" },
         { 15,  [pressure_3051clp],      "Pressure" },
         { 16,  [vortex_flow_8800],      "Flow          Vortex" },
         { 17,  [htg_3201],              "Level         Hydrostatic" },
         { 18,  [pressure_1152s] },
         { 19,  [htg_3202],              "Level         Hydrostatic" },
         { 20,  [pressure_3001s],        "Pressure" },
         { 21,  [multi_3095FT] },
         { 22,  [mv_3095],               "Flow          Multivariable Mass" },
         { 23,  [temperature_544] },
         { 24,  [temperature_644],       "Temperature" },
         { 25,  [temperature_3144],      "Temperature" },
         { 26,  [temperature_3244],      "Temperature   Multivariable" },
         { 27,  [level_3700] },
         { 28,  [level_3750],            "Level" },
         { 29,  [TRI_LOOP],              "Specialty     HART-to-Analog Signal Conv" },
         { 30,  [c_3095],                "Level" },
         { 31,  [mm_3095] },
         { 32,  [tank_3701] },
         { 33,  [level_3300],            "Level         Radar" },
         { 34,  [asu_3702] },
         { 35,  [pressure_2088s],        "Pressure" },
         { 36,  [probar],                "Flow          dP" },
         { 37,  [massprobar] },
         { 38,  [prov] },
         { 39,  [pressure_2090s],        "Pressure" },
         { 40,  [probar_uc],             "Flow          dP" }, /* Probar UC */
         { 41,  [d_2055] }, /* 2055D */
         { 42,  [pv_3095] }, /* 3095PV */
         { 45,  [sentry] }  /* Sentry */
         { 47,  [proplate_uc],           "Flow          dP" },
         { 54,  [pressure_1810],         "Pressure" },
         { 55,  [mag_flow_8712d],        "Flow          Magnetic" },
         { 56,  [pressure_4600],         "Pressure" },
         { 59,  [temperature_248],       "Temperature" },
         { 60,  [mag_flow_8732d],        "Flow          Magnetic" },
         { 64,  [pressure_2088lp],       "Pressure      Low Power" },
         { 65,  [level_5600] },
         { 66,  [presssure_951],         "Pressure" },
         { 67,  [level_5400],            "Level         Radar" },
         { 68,  [pressure_3051s_sis],    "Pressure      Certified SIL2" },
         { 69,  [temperature_3144p_sis], "Temperature" },
         { 71,  [pressure_4500],         "Pressure" },
         { 128, [generic],               "Interoperable" }

      CASE 39:  // PEEK MEASUREMENT
         {0, "FD900"}

      CASE 40:  // SCHLUMBERGER
         {1, "NEXGEN"}

      CASE 42:  // SIEMENS
         { 1,  [micro_k] },
         { 2,  [sitrans_l] },
         { 3,  [sipan_ph] },
         { 4,  [sitrans_fus],        "Flow          Ultrasonic" },
         { 5,  [sipan_lf] },
         { 6,  [sipan_o2] },
         { 7,  [sitrans_lr],         "Level         Radar" },
         { 10, [sitrans_phs],        "Pressure" },
         { 11, [sitrans_pds],        "Pressure" },
         { 12, [sitrans_pes],        "Pressure" },
         { 13, [sitrans_pms],        "Pressure" },
         { 15, [sitrans_t2k] },
         { 16, [sitrans_tw],         "Temperature" },
         { 18, [sitrans_tkh],        "Temperature" },
         { 19, [sitrans_th300],      "Temperature" },
         { 21, [sipart_ps2],         "Valve         Positioner/Electropneumatic" },
         { 22, [sitrans_fm],         "Flow          Electromagnetic" },
         { 23, [sitrans_fva] },
         { 24, [mag6000],            "Flow          Magnetic Inductive" },
         { 26, [sitrans_fm_it2_tm2], "Flow" },
         { 29, [sitrans_p300],       "Pressure" }

      CASE 44:  // TOSHIBA
         { 2,  [ap3100], "Pressure" },
         { 3,  [ap3110] },
         { 4,  [ap3120] },
         { 5,  [ap3140] },
         { 6,  [ap3150] },
         { 7,  [ap3170] },
         { 8,  [ap3180] },
         { 9,  [ap3190] },
         { 10, [lf220], "Flow          Magnetic" }

      CASE 46:  // ROSEMOUNT ANALYTICAL
         { 6,  [ph_1181],              "Analytical    pH" },
         { 7,  [conductivity_1181],    "Analytical    Conductivity" },
         { 10, [ph_1054] },
         { 11, [conductivity_1054] },
         { 12, [oxymitter_4000],       "Analytical    Oxygen" },
         { 13, [wc_3000],              "Analytical" },
         { 14, [ra3081fg],             "Analytical" },
         { 15, [opm_2000r],            "Analytical    Opacity Monitor" },
         { 16, [ocx_4000],             "Analytical    Oxygen/Combustion" },
         { 20, [ph_3081],              "Analytical    pH" },
         { 21, [c_3081],               "Analytical    2-wire Conductivity" },
         { 22, [ra3081_81con],         "Analytical    2-wire Conductivity w/ pure H20" },
         { 23, [ra3081_81t],           "Analytical    2-wire Conductivity w/ % conc" },
         { 24, [ocx_cl_oz_5081a],      "Analytical    2-wire Oxygen/Chlorine/Ozone" },
         { 25, [conductivity_5081c_t], "Analytical    2-wire Conductivity" },
         { 26, [ph_5081],              "Analytical    2-wire pH/ORP" },
         { 27, [xmt_a],                "Analytical    2-wire Oxygen/Chlorine/Ozone" },
         { 28, [xmt_c_t],              "Analytical    2-wire Contacting/Toroidal" },
         { 29, [xmt_ph],               "Analytical    2-wire pH/ORP" },
         { 30, [oxygen_5081fg],        "Analytical    Oxygen - High Temperature" },
         { 80, [ph_orp_54],            "Analytical    pH/ORP" },
         { 81, [c_54],                 "Analytical    Conductivity w/ % conc" }
         { 82, [ra52eph_orp],          "Analytical    pH/ORP/ISE" },
         { 83, [ra54ea],               "Analytical    Amperometric Analyzer" }

      CASE 47:  // VALMET
         { 1,  [psmart],    "Pressure      Pressure/Level" },
         { 60, [smartpulp], "Analytical    Consistency" },
         { 61, [smartlx],   "Analytical    Consistency" },
         { 62, [mcai],      "Analytical    Microwave-Pulp Consistency" },
         { 63, [smartlc],   "Analytical    Consistency" }

      CASE 48:  // FLOWSERVE
         { 1, [logix_12xx],   "Valve         Digital Positioner" },
         { 2, [kemmer_c2100], "Valve         Positioner" },
         { 3, [logix_520],    "Valve         Digital Positioner" },
         { 4, [Logix_3200iq], "Valve         Digital Positioner" }

      CASE 50:  // VIATRAN
         { 1,  [viatran970],          "Pressure" },
         { 46, [viatran_ia_pressure], "Pressure" }

      CASE 54:  // YAMATAKE HONEYWELL
         { 1, [magnew],     "Flow          Magnetic" },
         { 2, [st3000],     "Pressure" },
         { 3, [svp],        "Valve         Smart Valve Positioner" },
         { 4, [thermoplus], "Temperature" },
         { 5, [ptg],        "Pressure" },
         { 8, [magnew_2w],  "Flow          Electromagnetic" }

      CASE 55:  // YOKOGAWA
         { 1,  [yewflo],   "Flow          Vortex" },
         { 2,  [yt200],    "Temperature   Differential" },
         { 3,  [unicom],   "Pressure" },
         { 4,  [eja],      "Pressure      Differential" },
         { 5,  [admag_ae], "Flow          Magnetic" },
         { 6,  [am11],     "Flow" },
         { 8,  [admag_se], "Flow" },
         { 9,  [yta],      "Temperature" },
         { 10, [yta70e],   "Temperature" },
         { 11, [dyf],      "Flow          Vortex" },
         { 12, [zr202],    "Analytical    Oxygen" },
         { 13, [zr402],    "Analytical    Oxygen" },
         { 20, [isc202],   "Analytical    Conductivity - Inductive" },
         { 21, [ph202],    "Analytical    pH" },
         { 22, [sc202],    "Analytical    Conductivity" },
         { 24, [do202],    "Analytical    Dissolved Oxygen" },
         { 39, [ph150],    "Analytical    pH Controller" },
         { 40, [sc150],    "Analytical    Conductivity/Resistivity Controller" },
         { 43, [ph450],    "Analytical    pH" },
         { 44, [sc450],    "Analytical    Conductivity/Resistivity" },
         { 45, [isc450],   "Analytical    Conductivity - Inductive" },
         { 64, [rotamass], "Flow          Mass" },
         { 65, [ramc],     "Flow          Variable Area" },
         { 66, [rcct_f3],  "Flow          Mass"},
         { 80, [axfa11],   "Flow          Magnetic" }
         { 81, [ejx],      "Pressure" },
         { 82, [axfa14],   "Flow          Magnetic" },
         { 83, [av550g],   "Analytical    Oxygen" },
         { 84, [ejx_mv],   "Pressure      Multivariable" }

      CASE 58:  // EXAC
         {211, "8320"},
         {212, "SX5100"}

      CASE 59:  // KDG MOBREY
         { 19, [solartron4301], "Pressure      w/ PID" },
         { 20, [solartron3301] },
         { 21, [msp100],        "Level" },
         { 33, [level_3300],    "Level         Radar" },
         { 41, [mlt100],        "Level         Displacement" },
         { 42, [solartron4291] },
         { 43, [msm400] },
         { 44, [msl600] },
         { 45, [mcu900] },
         { 46, [msp900_400] },
         { 47, [mrl700],        "Level         Radar" },
         { 48, [mrl800] },
         { 49, [mrl850] }

      CASE 61:  // PRINCO
         {1, "L4610"}

      CASE 62:  // SMAR
         { 1, [ld301], "Pressure      w/ PID" },
         { 2, [tt301], "Temperature   w/ PID" },
         { 3, [fy301], "Valve         Positioner/Electropneumatic" },
         { 4, [ld291], "Pressure" },
         { 5, [tp301], "Valve         Positioner" },
         { 6, [dt301], "Analytical    Density" }

      CASE 63:  // ECKARDT
         { 1, [tsv175],   "Temperature" },
         { 2, [dmu130],   "Pressure" },
         { 3, [ti_rtt20], "Temperature" },
         { 4, [srd991],   "Valve         Positioner" },
         { 5, [dmu140],   "Pressure" },
         { 6, [srd960],   "Valve         Universal Positioner" }

      CASE 66:  // SAMSON
        { 239, [samson373x_3], "Valve         Positioner" },
        { 249, [samson3780],   "Valve         Controller" }

      CASE 67:  // SPARLING INSTRUMENTS
           { 238, [fm6xx], "Flow          Magnetic" }

      CASE 69:  // KROHNE
         { 226, [esk2a],         "Flow          Variable Area" },
         { 227, [ifc300],        "Flow          Electromagnetic" },
         { 228, [optiflex1300c], "Level         Radar" },
         { 229, [optiwave7300c], "Level         Radar" },
         { 230, [m8e],           "Flow          Variable Area" },
         { 231, [ufc030],        "Flow          Ultrasonic" },
         { 232, [mfc05x],        "Flow          Coriolis Mass" },
         { 233, [ifc040],        "Flow          Electromagnetic" },
         { 234, [m10],           "Flow          Variable Area" },
         { 235, [ifc210] },
         { 236, [bm102],         "Level         Radar - Reflex" },
         { 237, [vfm31],         "Flow          Vortex" },
         { 238, [bm100],         "Level         Radar - Reflex" },
         { 239, [ifc020] },
         { 240, [vfm1091] },
         { 242, [eskii],         "Flow          Variable Area" },
         { 243, [ifc110],        "Flow          Magnetic" },
         { 244, [ifc090],        "Flow          Magnetic"  },
         { 245, [ufc500],        "Flow          Ultrasonic" },
         { 246, [ifc010] },
         { 247, [mfc08x],        "Flow          Coriolis Mass" },
         { 248, [ifc080],        "Flow          Magnetic" },
         { 249, [bm70],          "Level         Radar" }

      CASE 77:  // WESTLOCK CONTROLS
         { 1, [icot],       "Valve         Digital Positioner w/ PID" },
         { 2, [smartcal],   "Valve         Intelligent Positioner" },
         { 3, [d_epic],     "Valve         Digital Positioner" },
         { 4, [d_epic_esd], "Valve         Digital Positioner"}

      CASE 78:  // DREXELBROOK
           { 2, [universal_iii], "Level" }

      CASE 79:  // SAAB TANK CONTROL
         { 237, [saab_5600],    "Level         Radar" },
         { 239, [tankradarpro], "Level         Radar" }

      CASE 82:  // DRAEGER
         { 235, [polytron7000], "Analytical    Gas Detector" }
         { 237, [polytron2_ir], "Analytical    Infrared Gas Detector" },
         { 247, [polytron2],    "Analytical    Electrochemical Gas Detector" }

      CASE 86:  // MAGNETROL
         { 229, [model_705_3x],    "Level" },
         { 230, [model_rx5],    "Level" },
         { 231, [model_704],    "Level" },
         { 232, [jupiter],      "Level         Magnostrictive" },
         { 233, [model_ta2],    "Flow          Mass" },
         { 234, [model_707],    "Level" },
         { 235, [model_705_2x], "Level" },
         { 236, [model_708],    "Level" },
         { 237, [model_805],    "Level" },
         { 238, [model_705],    "Level" },
         { 239, [smartez],      "Level" }

      CASE 87:  // NELES JAMESBURY
         {239, "ND800"},
         {238, "ND800 PT"}

      CASE 88:  // MILLTRONICS
         { 200, [probe_86x],     "Level         Ultrasonic" },
         { 201, [sitrans_lr300], "Level         Radar - Pulse" },
         { 202, [lr_200],        "Level         Radar" },
         { 204, [sitrans_lr400], "Level         Radar" },
         { 205, [probe_lu6m],    "Level         Ultrasonic"},
         { 206, [probe_lu12m],   "Level         Ultrasonic"},
         { 207, [probe_lr],      "Level         Radar"}
         
      CASE 90:  // ANDERSON
         {200, "ANDRSN1"}

      CASE 91:  // INOR
         {239, "MESO"}

      CASE 92:  // ROBERTSHAW
         {200, "EXCALIBUR 7000"}

      CASE 94:  // ACCUTECH
         {239, "AI1500"}

      CASE 96:  // KAMSTRUP
         {238, "FLEXBAR HRT"},
         {239, "FLEXTOP HRT"}

      CASE 97:  // KNICK
         { 232, [condi_2211], "Analytical    Conductivity" },
         { 233, [oxy_2211],   "Analytical    Dissolved Oxygen" },
         { 234, [cond_2211],  "Analytical    Conductivity" },
         { 235, [ph_2211],    "Analytical    pH" }

      CASE 101:  // MASONEILAN
         { 100, [hdlt], "Level" },
         { 200, [svi],  "Valve         Positioner" },
         { 201, [svi2], "Valve         Positioner" }

      CASE 102:  // BESTA
         {239, "CLS420"}

      CASE 103:  // OHMART
         {224, "XMTR"}

      CASE 104:  // HAROLD BECK AND SONS
         {1, "ESR D"}

      CASE 107:  // WIKA
         {239, "T32H"}

      CASE 108:  // BOPP AND REUTHER HEINRICHS
         { 235, [umc2], "Flow          Mass" },
         { 236, [es],   "Flow          Electronic" },
         { 237, [vtx],  "Flow          Vortex" },
         { 238, [dimf], "Analytical    Density" },
         { 239, [ust1], "Flow" }

      CASE 109:  // PR ELECTRONICS
         { 238, [pretrans_6335], "Temperature" },
         { 239, [pretop_5335],   "Temperature" }

      CASE 113:  // APPARATEBAU HUNDSBACH
         {238, "MT115"}

      CASE 114:  // DYNISCO
         {224, "IPX"}

      CASE 120:  // BURKERT
           { 239, [burkert8630] }

      CASE 124:  // GLI
         { 238, [m33_53p] },
         { 239, [m33_53c] }

      CASE 126:  // PMC
         {224, "SMT EL"}

      CASE 127:  // LABOM
        { 60,  [pascal_ci], "Pressure" },
        { 224, [ipx],       "Pressure      Multivariable" }

      CASE 129:  // TURBO
           { 127, [intermag_transmag], "Flow          Magnetic" }

      CASE 131:  // SMC
         {239, "F793 E701"}

      CASE 133:  // HUAKONG
         {127, "HK TT01"}

      CASE 153:   // METRAN
         { 124, [metran_300prm] },
         { 125, [metran_49] },
         { 126, [metran_280] },
         { 127, [metran_100] }
         
      CASE 206:   // ROOST
         { 128, [epp300] }

      CASE 239:  // HCF
         {235, "STD"},
         {236, "SAMPLE4"},
         {237, "SAMPLE3"},
         {238, "SAMPLE2"},
         {239, "SAMPLE1"}
      }

      { 252, [unknown] }
   }
}

// --- IdentCheck METHODs ------------------------------------------

VARIABLE deviceIdTmp
{
   LABEL "Device ID";
   TYPE UNSIGNED_INTEGER(3);
   CLASS LOCAL;
}

METHOD   std_CheckDeviceIdentification
{
   LABEL    [CheckDeviceIdentification];
   HELP     [deviceidentification_help];
   CLASS    INPUT;
   ACCESS      OFFLINE;
   DEFINITION
   {
      int  iResult ;
      char szTrace[] ;
      char sz[] ;
      char sz1[] ;
      char sz2[] ;
      char sz3[] ;
      char sz4[] ;
      char sz5[] ;
      char szChoice[] ;
      char szTag[];
      char szTagStored[];
      long var_id[10] ;
      long lDevice_Id ;
      long var_idDevice_Id;
      int iSelection;

      int bOK ;

      //ACKNOWLEDGE("Check Generic");
      ret_CheckDeviceIdentification = ID_CHECK_ABORT ;
      ret_CheckTAGIdentity = 1;

      _TRACE("std_CheckDeviceIdentification") ;

      var_idDevice_Id = VARID(device_id) ;

      if( var_idDevice_Id != 0 )
      {
         lDevice_Id = lvar_value(device_id) ;
         deviceIdTmp = device_id;

      }
      else
      {
         lDevice_Id = lvar_value(final_assembly_number) ;
         var_idDevice_Id = VARID(final_assembly_number) ;
         szTrace = "*** Rev 4 DD (final_assembly_number (VARID " 
                  + itoa(var_idDevice_Id)
                  + ") used) !" ;
         _TRACE(szTrace) ;

         deviceIdTmp = final_assembly_number;
      }

      bOK = 1 ;

      iResult = ID_FC_ERROR;
      while (iResult >= ID_FC_ERROR)
      {
         id_command_num = COMMAND_ID_READ_UNIQUE_IDENTIFIER;
         iResult = ReadCommand(id_read_unique_identifier) ;

         if( iResult < ID_FC_ERROR )
         {
            id_command_num = COMMAND_ID_READ_LONG_TAG;
            iResult = ReadCommand(id_read_long_tag);
         }
         id_command_num = id_command_num.DEFAULT_VALUE;

         if (iResult >= ID_FC_ERROR)
         {
            if (id_Method_In_SilentMode == 0)
               ACKNOWLEDGE([IdentCheckCommProblem]);
            else
               process_abort();
         }
      }

      sz1 = id_longTag ;
      sz1 = strtrim(sz1);
      szTrace = "Device found: Man " 
               + itoa(ivar_value(id_manufacturer_id))
               + " DeviceType "
               + itoa(ivar_value(id_device_type))
               + " DeviceRevision " 
               + itoa(ivar_value(id_device_revision)) 
               + " SoftwareRevision " 
               + itoa(ivar_value(id_software_revision)) 
               + " DeviceID "
               + itoa(lvar_value(id_device_id)) 
               + " Long TAG "
               + sz1 ;

      _TRACE(szTrace) ;

      iassign(dd_manufacturer_id, GET_MANUFACTURER()) ;
      iassign(dd_device_type, GET_DEVICE_TYPE()) ;
      iassign(dd_device_revision, GET_DEVICE_REVISION()) ;
      
      szTrace = "Device expected: Man " 
               + itoa(ivar_value(dd_manufacturer_id))
               + " DeviceType "
               + itoa(ivar_value(dd_device_type))
               + " DeviceRevision " 
               + itoa(ivar_value(dd_device_revision)) 
               + " DeviceID "
               + itoa(lDevice_Id) ;

      _TRACE(szTrace) ;

      iassign(dd_manufacturer_id, GET_MANUFACTURER()) ;
      iassign(dd_device_type, GET_DEVICE_TYPE()) ;
      iassign(dd_device_revision, GET_DEVICE_REVISION()) ;
      
      var_id[0] = VARID(dd_manufacturer_id) ;
      var_id[1] = VARID(dd_device_type) ;
      var_id[2] = VARID(dd_device_revision) ;
      var_id[3] = VARID(id_manufacturer_id) ;
      var_id[4] = VARID(id_device_type) ;
      var_id[5] = VARID(id_device_revision) ;
      var_id[6] = VARID(id_longTag) ;

      if( (GET_MANUFACTURER() != 0) && (GET_MANUFACTURER() != 252) )
      {
         if( (ivar_value(id_manufacturer_id) != GET_MANUFACTURER())
            || (ivar_value(id_device_type) != GET_DEVICE_TYPE())
            || (ivar_value(id_device_revision) < ivar_value(dd_device_revision))
          )
         {
            bOK = 0 ;
            if (id_Method_In_SilentMode == 1)
            {
               LOG_MESSAGE(0, [DeviceTypeMismatch]) ;
            }
            else
            {
               put_message([DeviceTypeMismatch2], var_id) ;
            }
            process_abort() ;
         }
      }

      if( bOK && (lDevice_Id != 0) && (lDevice_Id != 1) )
      {
         if( id_device_id != lvar_value(device_id) )
         {
            iSelection = 0;

            if (id_Method_In_SilentMode == 1)
            {
               LOG_MESSAGE(0, [DeviceIdMismatch]) ;
            }
            else
            {
               iSelection = SELECT_FROM_LIST([DeviceIdMismatch3],  [OptionsDeviceIdStore]) ;
            }

            if( iSelection == 1 )
            {
               if( VARID(device_id) != 0 )
               {
                  lassign(device_id, lvar_value(id_device_id)) ;
               }
               else
               {
                  lassign(final_assembly_number, lvar_value(id_device_id)) ;
               }
            }
            else
            {
               bOK = 0;
            }
         }
         
         if( bOK )
         {
            sz1 = longTag;
            sz1 = strtrim(sz1);
            szTag = id_longTag;
            szTag = strtrim(szTag);
            
            if( strtrim(szTag) != strtrim(sz1) )
            {
               if (id_Method_In_SilentMode == 1)
               {
                  ret_CheckTAGIdentity = 0;
                  LOG_MESSAGE(0, LONGTAG_MISMATCH_3) ;
               }
               else
               {
                  iSelection = SELECT_FROM_LIST(LONGTAG_MISMATCH_3,  [CancelContinue]) ;
                  if( iSelection == 0 )
                  {
                     bOK = 0;
                  }
               }
            }
         }
      }

      if( bOK )
      {
         ret_CheckDeviceIdentification = ID_CHECK_SUCCESS ;
      }
   }
}

METHOD   std_CheckDeviceIdentificationUpload
{
   LABEL    [CheckDeviceIdentificationUpload];
   HELP     [deviceidentification_help];
   CLASS    INPUT;
   ACCESS      OFFLINE;
   DEFINITION
   {
      int  iResult ;
      char szTrace[] ;
      char sz[] ;
      char sz1[] ;
      char sz2[] ;
      char sz3[] ;
      char sz4[] ;
      char sz5[] ;
      char szChoice[] ;
      char szTag[];
      char szTagStored[];
      long var_id[10] ;
      int bContinue ;
      int bOK ;
      int iChoice ;
      int iSelection;

      long lDevice_Id ;
      long var_idDevice_Id ;

      //ACKNOWLEDGE("Check Upload");

      ret_CheckDeviceIdentification = ID_CHECK_ABORT ;
      ret_CheckTAGIdentity = 1;

      _TRACE("std_CheckDeviceIdentificationUpload") ;

      var_idDevice_Id = VARID(device_id) ;

      if( var_idDevice_Id != 0 )
      {
         lDevice_Id = lvar_value(device_id) ;
         deviceIdTmp = device_id;
      }
      else
      {
         lDevice_Id = final_assembly_number ;
         var_idDevice_Id = VARID(final_assembly_number) ;
         szTrace = "*** Rev 4 DD (final_assembly_number (VARID " 
                  + itoa(var_idDevice_Id)
                  + ") used) !" ;
         _TRACE(szTrace) ;
         deviceIdTmp = final_assembly_number;
      }

      bOK = 1 ;

      iResult = ID_FC_ERROR;
      while (iResult >= ID_FC_ERROR)
      {
         id_command_num = COMMAND_ID_READ_UNIQUE_IDENTIFIER;
         iResult = ReadCommand(id_read_unique_identifier) ;
   
         if( iResult < ID_FC_ERROR )
         {
            id_command_num = COMMAND_ID_READ_LONG_TAG;
            iResult = ReadCommand(id_read_long_tag);
         }
         id_command_num = id_command_num.DEFAULT_VALUE;

         if (iResult >= ID_FC_ERROR)
         {
            if (id_Method_In_SilentMode == 0)
               ACKNOWLEDGE([IdentCheckCommProblem]);
            else
               process_abort();
         }
      }

      sz1 = id_longTag ;
      sz1 = strtrim(sz1);
      sz2 = itoa(ivar_value(id_write_protect)) ;
      szTrace = "Device found: Man " 
               + itoa(ivar_value(id_manufacturer_id))
               + " DeviceType "
               + itoa(ivar_value(id_device_type))
               + " DeviceRevision " 
               + itoa(ivar_value(id_device_revision)) 
               + " DeviceID "
               + itoa(lvar_value(id_device_id)) 
               + " Long TAG "
               + sz1 ;

      _TRACE(szTrace) ;

      iassign(dd_manufacturer_id, GET_MANUFACTURER()) ;
      iassign(dd_device_type, GET_DEVICE_TYPE()) ;
      iassign(dd_device_revision, GET_DEVICE_REVISION()) ;
      
      szTrace = "Device expected: Man " 
               + itoa(ivar_value(dd_manufacturer_id))
               + " DeviceType "
               + itoa(ivar_value(dd_device_type))
               + " DeviceRevision " 
               + itoa(ivar_value(dd_device_revision)) 
               + " DeviceID "
               + itoa(lDevice_Id) ;

      _TRACE(szTrace) ;
      
      var_id[0] = VARID(dd_manufacturer_id) ;
      var_id[1] = VARID(dd_device_type) ;
      var_id[2] = VARID(dd_device_revision) ;
      var_id[3] = VARID(id_manufacturer_id) ;
      var_id[4] = VARID(id_device_type) ;
      var_id[5] = VARID(id_device_revision) ;
      var_id[6] = VARID(id_longTag) ;

      if( (GET_MANUFACTURER() != 0) && (GET_MANUFACTURER() != 252) )
      {
         if( (ivar_value(id_manufacturer_id) != GET_MANUFACTURER())
            || (ivar_value(id_device_type) != GET_DEVICE_TYPE())
            || (ivar_value(id_device_revision) < ivar_value(dd_device_revision))
          )
         {
            bOK = 0 ;
            if (id_Method_In_SilentMode == 1)
            {
               LOG_MESSAGE(0, [DeviceTypeMismatch]) ;
            }
            else
            {
               put_message([DeviceTypeMismatch2], var_id) ;
            }
            process_abort() ;
         }
      }

      if( bOK && (lDevice_Id != 0) && (lDevice_Id != 1) )
      {
         if( id_device_id != lDevice_Id )
         {
            iSelection = 0;

            if (id_Method_In_SilentMode == 1)
            {
               LOG_MESSAGE(0, [DeviceIdMismatch]) ;
            }
            else
            {
               iSelection = SELECT_FROM_LIST([DeviceIdMismatch3],  [OptionsDeviceIdStore]) ;
            }

            if( iSelection == 1 )
            {
               if( VARID(device_id) != 0 )
               {
                  lassign(device_id, lvar_value(id_device_id)) ;
               }
               else
               {
                  lassign(final_assembly_number, lvar_value(id_device_id)) ;
               }
            }
            else
            {
               bOK = 0;
            }
         }
      }
      else
      {
         if( VARID(device_id) != 0 )
         {
            lassign(device_id, lvar_value(id_device_id)) ;
         }
         else
         {
            lassign(final_assembly_number, lvar_value(id_device_id)) ;
         }
      }

#ifdef CHECK_SOFTWARE_REVISION
      if( bOK )
      {
         if( software_revision > id_software_revision )
         {
            var_id[0] = VARID(id_software_revision) ;
            var_id[1] = VARID(software_revision) ;

            if (id_Method_In_SilentMode == 0)
            {
               acknowledge([SoftwareRevisionToLow], var_id) ;
            }

            // user clicked 'OK': store new software_revision in offline data
            iassign(software_revision, ivar_value(id_software_revision)) ;
         }
      }
#endif

      if( bOK && (id_firstCheck == 0) )
      {
         szTag = id_longTag;
         szTag = strtrim(szTag);
         szTagStored = longTag;
         szTagStored = strtrim(szTagStored);

         if( szTag != szTagStored )
         {
            ret_CheckTAGIdentity = 0;

            if (id_Method_In_SilentMode == 1)
            {
               ret_CheckTAGIdentity = 0;
               LOG_MESSAGE(0, LONGTAG_MISMATCH) ;
            }
            else
            {
               get_dictionary_string(cancel, sz1) ;
               sz2 = LONGTAG_MISMATCH_KEEP_OLD;
               sz3 = LONGTAG_MISMATCH_APPLY;

               sz4 = longTag ;
               sz5 = id_longTag ;
               sz = sz1
                  + ";"
                  + sz2
                  + " (" 
                  + sz4 
                  + ")" 
                  + ";" 
                  + sz3
                  + " (" 
                  + sz5 
                  + ")" ;

               iChoice = SELECT_FROM_LIST(LONGTAG_MISMATCH, sz) ;

               switch (iChoice)
               {
                  case 0:                 // abort
                     bOK = 0 ;
                     break;

                  case 1:                 // keep old
                     break;

                  default:             // use new
                     longTag = id_longTag ;
                     break;
               }
            }
         }
      }

      if( bOK )
      {
         id_firstCheck = 0 ;
         ret_CheckDeviceIdentification = ID_CHECK_SUCCESS ;
         _TRACE("IdentCheck was succesful !") ;
      }
   }
}

METHOD   std_CheckDeviceIdentificationDownload
{
   LABEL    [CheckDeviceIdentificationDownload];
   HELP     [deviceidentification_help];
   CLASS    INPUT;
   ACCESS      OFFLINE;
   DEFINITION
   {
      int  iResult ;
      char szTrace[] ;
      char sz[] ;
      char sz1[] ;
      char sz2[] ;
      char sz3[] ;
      char sz4[] ;
      char sz5[] ;
      char szChoice[] ;
      long var_id[10] ;
      int bContinue ;
      int bOK ;
      long lDevice_Id ;
      long var_idDevice_Id ;
      int iSelection;

      //ACKNOWLEDGE("Check Download");

      ret_CheckDeviceIdentification = ID_CHECK_ABORT ;
      ret_CheckTAGIdentity = 1;

      _TRACE("std_CheckDeviceIdentificationDownload") ;

      var_idDevice_Id = VARID(device_id) ;

      if( var_idDevice_Id != 0 )
      {
         lDevice_Id = lvar_value(device_id) ;
         deviceIdTmp = device_id;
      }
      else
      {
         lDevice_Id = final_assembly_number ;
         var_idDevice_Id = VARID(final_assembly_number) ;
         szTrace = "*** Rev 4 DD (final_assembly_number (VARID " 
                  + itoa(var_idDevice_Id)
                  + ") used) !" ;
         _TRACE(szTrace) ;

         deviceIdTmp = final_assembly_number;
      }

      bOK = 1 ;

      id_command_num = COMMAND_ID_READ_UNIQUE_IDENTIFIER;
      iResult = ReadCommand(id_read_unique_identifier) ;

      if( iResult < ID_FC_ERROR )
      {
         id_command_num = COMMAND_ID_READ_LONG_TAG;
         iResult = ReadCommand(id_read_long_tag);
      }
      id_command_num = id_command_num.DEFAULT_VALUE;

      if (id_Method_In_SilentMode == 0)
      {
         while( iResult >= ID_FC_ERROR )
         {
            ACKNOWLEDGE([IdentCheckCommProblem]) ;
      
            id_command_num = COMMAND_ID_READ_UNIQUE_IDENTIFIER;
            iResult = ReadCommand(id_read_unique_identifier) ;

            if( iResult < ID_FC_ERROR )
            {
               id_command_num = COMMAND_ID_READ_LONG_TAG;
               iResult = ReadCommand(id_read_long_tag);
            }
            id_command_num = id_command_num.DEFAULT_VALUE;
         }
      }

      sz1 = id_longTag ;
      sz1 = strtrim(sz1);
      szTrace = "Device found: Man " 
               + itoa(ivar_value(id_manufacturer_id))
               + " DeviceType "
               + itoa(ivar_value(id_device_type))
               + " DeviceRevision " 
               + itoa(ivar_value(id_device_revision)) 
               + " DeviceID "
               + itoa(lvar_value(id_device_id)) 
               + " Long TAG "
               + sz1 ;

      _TRACE(szTrace) ;

      iassign(dd_manufacturer_id, GET_MANUFACTURER()) ;
      iassign(dd_device_type, GET_DEVICE_TYPE()) ;
      iassign(dd_device_revision, GET_DEVICE_REVISION()) ;
      
      szTrace = "Device expected: Man " 
               + itoa(ivar_value(dd_manufacturer_id))
               + " DeviceType "
               + itoa(ivar_value(dd_device_type))
               + " DeviceRevision " 
               + itoa(ivar_value(dd_device_revision)) 
               + " DeviceID "
               + itoa(lDevice_Id) ;

      _TRACE(szTrace) ;

      var_id[0] = VARID(dd_manufacturer_id) ;
      var_id[1] = VARID(dd_device_type) ;
      var_id[2] = VARID(dd_device_revision) ;
      var_id[3] = VARID(id_manufacturer_id) ;
      var_id[4] = VARID(id_device_type) ;
      var_id[5] = VARID(id_device_revision) ;
      var_id[6] = VARID(id_longTag) ;

      if( (GET_MANUFACTURER() != 0) && (GET_MANUFACTURER() != 252) )
      {
         if( (ivar_value(id_manufacturer_id) != GET_MANUFACTURER())
            || (ivar_value(id_device_type) != GET_DEVICE_TYPE())
            || (ivar_value(id_device_revision) < ivar_value(dd_device_revision))
          )
         {
            bOK = 0 ;
            if (id_Method_In_SilentMode == 1)
            {
               LOG_MESSAGE(0, [DeviceTypeMismatch]) ;
            }
            else
            {
               put_message([DeviceTypeMismatch2], var_id) ;
            }
            process_abort() ;
         }
      }

      if( bOK && (lDevice_Id != 0) && (lDevice_Id != 1) )
      {
         if( id_device_id != lDevice_Id )
         {
            iSelection = 0;

            if (id_Method_In_SilentMode == 1)
            {
               LOG_MESSAGE(0, [DeviceIdMismatch]) ;
            }
            else
            {
               iSelection = SELECT_FROM_LIST([DeviceIdMismatch3],  [OptionsDeviceIdStore]) ;
            }

            if( iSelection == 1 )
            {
               if( VARID(device_id) != 0 )
               {
                  lassign(device_id, lvar_value(id_device_id)) ;
               }
               else
               {
                  lassign(final_assembly_number, lvar_value(id_device_id)) ;
               }
            }
            else
            {
               bOK = 0;
            }
         }
      }

      if( bOK && (id_firstCheck == 0) )
      {
         bContinue = 0 ;
         sz1 = longTag;
         
         if( id_longTag != sz1 )
         {
            ret_CheckTAGIdentity = 0;

            var_id[0] = VARID(longTag) ;
            var_id[1] = VARID(id_longTag) ;
         
            if (id_Method_In_SilentMode == 1)
            {
               ret_CheckTAGIdentity = 0;
               LOG_MESSAGE(0, LONGTAG_MISMATCH) ;
            }
            else
            {
               iSelection = SELECT_FROM_LIST(LONGTAG_MISMATCH_3,  [CancelContinue]) ;
               if( iSelection == 0 )
               {
                  bOK = 0;
               }
            }
         }
      }

      if( bOK )
      {
         id_firstCheck = 0;
         ret_CheckDeviceIdentification = ID_CHECK_SUCCESS ;
         _TRACE("IdentCheck was succesful !") ;
      }
   }
}

// --- Checking Write Protection ------------------------------------------

#define  ID_CHECK_WRITABLE       0
#define  ID_CHECK_WRITE_PROTECTED   1

VARIABLE ret_CheckWriteProtection
{
   LABEL    "Write Protection Result";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (1)
   {
      DEFAULT_VALUE  ID_CHECK_WRITABLE ;
   }
   HANDLING READ;
}

METHOD   std_CheckWriteProtection
{
   LABEL    "std_CheckWriteProtection";
   CLASS    INPUT;
   ACCESS      OFFLINE;
   DEFINITION
   {
      long var_id[10] ;
      int  iResult ;
      char szTrace[] ;
      char sz[] ;

      ret_CheckWriteProtection = ID_CHECK_WRITABLE ;

      _TRACE("std_CheckWriteProtection") ;

      id_command_num = COMMAND_ID_READ_PV_OUTPUT_INFO;
      iResult = ReadCommand(id_read_pv_output_info);
      id_command_num = id_command_num.DEFAULT_VALUE;
      if( iResult >= ID_FC_ERROR )
      {
         iassign(id_write_protect, 252) ;
      }

      sz = itoa(ivar_value(id_write_protect)) ;
      szTrace = "Write protect = " + sz ;

      _TRACE(szTrace) ;


      if( id_write_protect == 1 )
      {
         ret_CheckWriteProtection = ID_CHECK_WRITE_PROTECTED ;

         id_command_num = COMMAND_ID_READ_LONG_TAG;
         iResult = ReadCommand(id_read_long_tag);
         id_command_num = id_command_num.DEFAULT_VALUE;

         var_id[0] = VARID(id_manufacturer_id) ;
         var_id[1] = VARID(id_device_type) ;
         var_id[2] = VARID(id_longTag) ;

         if (id_Method_In_SilentMode == 1)
         {
            LOG_MESSAGE(0, [DeviceWriteProtected]) ;
         }
         else
         {
            acknowledge([DeviceWriteProtected], var_id) ;
         }
         
         // user clicked 'OK' so write to device anyway ...
         ret_CheckWriteProtection = ID_CHECK_WRITABLE ;
      }

   }
}

/*******************************************************************************/
/*****   Commands  **************************************************************/
/*******************************************************************************/

VARIABLE id_command_num
{
   LABEL    "Command Number";
   CLASS    LOCAL;
   TYPE     INTEGER (2)
   {
      DEFAULT_VALUE  -1;
   }
   HANDLING READ;
}

COMMAND id_read_unique_identifier
{
   NUMBER id_command_num;           // NUMBER 0
   OPERATION READ;
   TRANSACTION
   {
      REQUEST
      {
      }
      REPLY
      {
         response_code, 
         device_status,
         254, 
         id_manufacturer_id, 
         id_device_type,  
         id_dummy_byte, 
         id_dummy_byte, 
         id_device_revision, 
         id_software_revision,
         id_dummy_byte, 
         id_dummy_byte, 
         id_device_id 
      }
   }
   RESPONSE_CODES
   {
      0, SUCCESS, [no_command_specific_errors];
      6, MISC_ERROR, [xmtr_specific_command_error];
      16,   MODE_ERROR, [access_restricted];
   }
}

COMMAND id_read_pv_output_info
{
   NUMBER id_command_num;           // NUMBER 15
   OPERATION READ;
   TRANSACTION
      {
         REQUEST
            {}

         REPLY
            {
               response_code, device_status,
               id_dummy_byte, id_dummy_byte,
               id_dummy_byte, id_dummy_float, id_dummy_float,
               id_dummy_float, id_write_protect, id_dummy_byte
            }
      }

   RESPONSE_CODES
      {
         0, SUCCESS, [no_command_specific_errors];
         6, MISC_ERROR, [xmtr_specific_command_error];
         16,   MODE_ERROR, [access_restricted];
      }
}

COMMAND id_read_long_tag
{
    NUMBER id_command_num;    // NUMBER 20;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, id_longTag
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        16, MODE_ERROR, [access_restricted];
        32, MODE_ERROR, [busy];
    }
}

// --------------------------------------------------------------------
// Handling Configuration Changed Flag

VARIABLE ret_ConfigChangedState
{
   LABEL    "ConfigChanged";
   CLASS    LOCAL;
   TYPE     UNSIGNED_INTEGER (1)
   {
      DEFAULT_VALUE  0 ;
   }
   HANDLING READ & WRITE;
}

METHOD   std_GetConfigChangedState
{
   LABEL    "std_GetConfigChangedState";
   ACCESS      OFFLINE;
   DEFINITION
   {
      ret_ConfigChangedState  = 0 ;

      _TRACE("std_GetConfigChangedState") ;

      id_command_num = COMMAND_ID_READ_UNIQUE_IDENTIFIER;
      ReadCommand(id_read_unique_identifier) ;
      id_command_num = id_command_num.DEFAULT_VALUE;

      _TRACE("std_GetConfigChangedState device_status = " + device_status) ;

      if( device_status & 0x40 )
      {
         ret_ConfigChangedState  = 1 ;
      }
      else
      {
         ret_ConfigChangedState  = 0 ;
      }
   }
}

METHOD   std_ResetConfigChangedState
{
   LABEL    [configuration_flag_reset];
   ACCESS      OFFLINE;
   DEFINITION
   {
      _TRACE("std_ResetConfigChangedState") ;
      send_command(38);

      ret_ConfigChangedState  = 0 ;
   }
}
