/*
$Id: hart/HART_Sensor_Level_Milltronics_LR560/tvtshaper120.inc 1.6 2009/11/16 12:12:09EST Bond, Roland (Roland Bond) Dev  $
$ProjectName: d:/elect/source/device_descriptions/pdm/potter_pdm.pj $   $Locker:  $
*/
#ifndef TVTSHAPER120_INC
#define TVTSHAPER120_INC
#endif
/*                                                       For Internal Use Only.
   Description ----------------------------------------------------------------
       Library File For 120 point TVT Shaper.

         This is an EDD feature to support SMPI Standard 1200 point manual TVT shapers

         The only method to use is "method_menu_tvt_shaper" the remaining items
         are support elements

         The basic structure:
                             method_menu_tvt_shaper
                                        |
                             menu_dialog_tvt_shaper
                                        |
                   -------------------------------------------------------------------------------------------
                   |               |                               |                       |                |
    menu_page_tvt_shaper_graph  menu_page_tvt_shaper_list1 ...tvt_shaper_list2  ...tvt_shaper_list3 ...tvt_shaper_list4
                        |
             ---------------------------------------------------
             |                        |                        |
      graph_tvt_shaper   menu_group_tvt_presettings     menu_group_tvt_shaper,


         Copyright (C) Siemens-Milltronics Process Instruments Inc.
         All Rights Reserved.

   Warnings -------------------------------------------------------------------

   Special Revision Notes -----------------------------------------------------
      To add this feature to a DD:

      1) Add "method_menu_tvt_shaper" under the "Device" menu
      2) Make sure the defines the options file reference the correct variables
      3) Make sure the dictionary entries from the options file are defined
      4) Include "tvtshaper40.inc" (this file) in the main ddl file
      5) Add "tvtshaper40.inc" (this file) to the devices file
      6) Add "tvtshaper40opt.dd" to the devices file

   Defines --------------------------------------------------------------------*/
#define NUMBER_TVT_SHAPER_POINTS                     120

// Tvt shaper modes
#define TVT_SHAPER_MODE_OFF 0
#define TVT_SHAPER_MODE_ON  1

/* Include Files --------------------------------------------------------------*/

/* Collections ----------------------------------------------------------------*/

/* Relations (Unit, Array, Refresh) -------------------------------------------*/
REFRESH refresh_tvt_shaper_array
{
   var_local_tvt_shaper_index_display
   :
   var_local_tvt_shaper_value_display
}

/* Variable Definitions -------------------------------------------------------*/
var_backup_tvt_shaper_mode     LIKE VARIABLE TVT_SHAPER_MODE  {}

ARRAY array_backup_tvt_shaper_points
{
   TYPE                par_db_start_y_axis;
   NUMBER_OF_ELEMENTS  NUMBER_TVT_SHAPER_POINTS;
}

/*-----------------------------------------------------------------------------
   Description: The index of the value currently being edited
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_profile_taken
{
   CLASS       LOCAL;
   TYPE        UNSIGNED_INTEGER(1);
   DEFAULT_VALUE 0;
}

/*-----------------------------------------------------------------------------
   Description: The index of the value currently being edited
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_index_display
{
   LABEL       [point_label];
   HELP        [point_help];
   CLASS       LOCAL;
   TYPE        INTEGER(1)
   {
      MIN_VALUE     1;
      MAX_VALUE     NUMBER_TVT_SHAPER_POINTS;
   }
   DEFAULT_VALUE 1;
	POST_EDIT_ACTIONS
	{
       method_get_tvt_shaper_index_display,
       method_set_tvt_shaper_offset_display
	}
   HANDLING    READ & WRITE;
}

/*-----------------------------------------------------------------------------
   Description: The value currently being edited
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_value_display
{
   LABEL       [offset_label];
   HELP        [offset_help];
   CLASS       LOCAL;
   TYPE        INTEGER(1)
   {
      MIN_VALUE     TVT_SHAPER_MINIMUM_VALUE * TVT_SHAPER_STEP_SIZE;
      MAX_VALUE     TVT_SHAPER_MAXIMUM_VALUE * TVT_SHAPER_STEP_SIZE;
   }
   DEFAULT_VALUE 0;
   PRE_EDIT_ACTIONS
	{
       method_set_tvt_shaper_offset_display
	}
	POST_EDIT_ACTIONS
	{
       method_get_tvt_shaper_offset_display
	}
   HANDLING    READ & WRITE;
   CONSTANT_UNIT [UNIT_DECIBEL];
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_x_axis_echo_start
{
   TYPE FLOAT;
   CLASS LOCAL;
}

VARIABLE var_local_tvt_shaper_tvt_start
{
   TYPE FLOAT;
   CLASS LOCAL;
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_x_axis_echo_increment
{
   TYPE FLOAT;
   CLASS LOCAL;
}

VARIABLE var_local_tvt_shaper_tvt_increment
{
   TYPE FLOAT;
   CLASS LOCAL;
}

VARIABLE var_local_tvt_shaper_zero_offset
{
   CLASS LOCAL;
   TYPE  FLOAT;
}

VARIABLE var_local_tvt_shaper_step_size
{
   CLASS LOCAL;
   TYPE  FLOAT;
}

VARIABLE var_local_marker_start
{
   CLASS LOCAL;
   TYPE  INTEGER(2);
   DEFAULT_VALUE 0;
}

VARIABLE var_local_marker_end
{
   CLASS LOCAL;
   TYPE  INTEGER(2);
   DEFAULT_VALUE 1;
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
ARRAY array_local_tvt_point
{
   TYPE                local_data_point_y;
   NUMBER_OF_ELEMENTS  NUMBER_TVT_POINTS;
}

/* Command Definitions --------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/

/* Menu Definitions -----------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description: Displays a tabbed page with a graphical editing interface
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU Page_tvt_shaper_graph
{
   LABEL      [menu_tvt_shaper_label];
   STYLE_PAGE
   ITEMS
   {
      COOP_IDENTITY_ON
      graph_tvt_shaper,
      [profile_legend_label],
      COLUMNBREAK,
      Group_tvt_shaper,
      Group_tvt_presettings,
      Group_tvt_shaper_mode
   }
}

/*-----------------------------------------------------------------------------
   Description: displays a group on the graph page for refreshing the profiles
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU Group_tvt_presettings
{
   LABEL      [grp_tvt_presettings_label];
   STYLE_GROUP
   ITEMS
   {
      method_tvt_shaper_update_from_device,
      COLUMNBREAK,
      method_tvt_shaper_transfer_to_device
   }
}

/*-----------------------------------------------------------------------------
   Description: displays a group on the graph page for editing the TVT
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU Group_tvt_shaper
{
   LABEL      [menu_tvt_shaper_label];
   STYLE_GROUP
   ITEMS
   {
      var_local_tvt_shaper_index_display,
      method_tvt_index_plus,
      method_tvt_index_minus,
      COLUMNBREAK,
      var_local_tvt_shaper_value_display,
      method_tvt_value_increase,
      method_tvt_value_decrease
   }
}

/*-----------------------------------------------------------------------------
   Description: displays the TVT Shaper Mode
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU Group_tvt_shaper_mode
{
   STYLE_GROUP
   ITEMS
   {
      par_tb_tvt_shaper_mode
   }
}

/* Visual component Definitions -----------------------------------------------*/

/*-----------------------------------------------------------------------------
   Description: Displays graph area, content is referenced from here
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
GRAPH graph_tvt_shaper
{
    HEIGHT     X_LARGE;
    WIDTH      LARGE;
    X_AXIS     x_axis_tvt_shaper;
    MEMBERS
    {
        WAVEFORM_TVT_SHAPER_MARKER_START, waveform_tvt_shaper_marker_start;
        WAVEFORM_TVT_SHAPER_MARKER_END,   waveform_tvt_shaper_marker_end;
        WAVEFORM_TVT_SHAPER_TVT,  waveform_tvt_shaper_tvt_profile;
        WAVEFORM_TVT_SHAPER_ECHO, waveform_tvt_shaper_echo_profile;
    }
    VALIDITY IF (var_local_tvt_shaper_profile_taken == 1)
             {
                TRUE;
             }
             ELSE
             {
                FALSE;
             }
}

WAVEFORM waveform_tvt_shaper_echo_profile
{
   Y_AXIS     y_axis_tvt_shaper;
   LINE_COLOR TVT_SHAPER_ECHO_PROFILE_COLOR;
   TYPE YT
   {
      X_INITIAL        var_local_tvt_shaper_x_axis_echo_start;
      X_INCREMENT      var_local_tvt_shaper_x_axis_echo_increment;
      NUMBER_OF_POINTS NUMBER_ECHO_POINTS;
      Y_VALUES
      {
         echo_profile_point
      }
   }
   INIT_ACTIONS
      {
      method_tvt_shaper_measure
      }
}

WAVEFORM waveform_tvt_shaper_tvt_profile
{
   Y_AXIS     y_axis_tvt_shaper;
   LINE_COLOR TVT_SHAPER_TVT_PROFILE_COLOR;
   TYPE YT
   {
      X_INITIAL        var_local_tvt_shaper_tvt_start;
      X_INCREMENT      var_local_tvt_shaper_tvt_increment;
      NUMBER_OF_POINTS NUMBER_TVT_POINTS;
      Y_VALUES
      {
         array_local_tvt_point
      }
   }
}

WAVEFORM waveform_tvt_shaper_marker_start
{
   Y_AXIS     y_axis_tvt_shaper;
   LINE_COLOR 0x000000;
   TYPE XY
   {
      X_VALUES
      {
         var_local_marker_start, var_local_marker_start
      }
      Y_VALUES
      {
         par_db_end_y_axis, par_db_start_y_axis 
      }
      NUMBER_OF_POINTS 2;
   }
}

WAVEFORM waveform_tvt_shaper_marker_end
{
   Y_AXIS     y_axis_tvt_shaper;
   LINE_COLOR 0x000000;
   TYPE XY
   {
      X_VALUES
      {
         var_local_marker_end, var_local_marker_end
      }
      Y_VALUES
      {
         par_db_end_y_axis, par_db_start_y_axis 
      }
      NUMBER_OF_POINTS 2;
   }
}

AXIS x_axis_tvt_shaper
{
   LABEL         [tvt_shaper_breakpoints_label];
   MIN_VALUE     var_local_tvt_shaper_x_axis_echo_start;
   MAX_VALUE     NUMBER_TVT_SHAPER_POINTS + 1;
   CONSTANT_UNIT [point_label];
}

AXIS y_axis_tvt_shaper
{
    LABEL [profile_label];
    MIN_VALUE par_db_start_y_axis;
    MAX_VALUE par_db_end_y_axis;
    CONSTANT_UNIT [UNIT_DECIBEL];
}

/* Method Definitions ---------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description: Make a backup copy of the data we are modifying so it can be
                restored if needed
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_backup_tvt_shaper
{
   DEFINITION
   {
      int point;

      // Make a local copy so we can restore data
      var_backup_tvt_shaper_mode = TVT_SHAPER_MODE;

      for( point = 0; point < NUMBER_TVT_SHAPER_POINTS; point++ )
      {
         array_backup_tvt_shaper_points[point] = TVT_SHAPER_ARRAY[point];
      }
   }
}

/*-----------------------------------------------------------------------------
   Description: Restore the data we are modifying from the backup copy
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_restore_tvt_shaper
{
   DEFINITION
   {
      int point;

      // Restore data from local copy
      TVT_SHAPER_MODE = var_backup_tvt_shaper_mode;

      for( point = 0; point < NUMBER_TVT_SHAPER_POINTS; point++ )
      {
         TVT_SHAPER_ARRAY[point] = array_backup_tvt_shaper_points[point];
      }
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_get_tvt_shaper_index_display
{
   DEFINITION
   {
      if (var_local_tvt_shaper_index_display > var_local_tvt_shaper_index_display.MAX_VALUE)
      {
         var_local_tvt_shaper_index_display = var_local_tvt_shaper_index_display.MAX_VALUE;
      }
      if (var_local_tvt_shaper_index_display < var_local_tvt_shaper_index_display.MIN_VALUE)
      {
         var_local_tvt_shaper_index_display = var_local_tvt_shaper_index_display.MIN_VALUE;
      }
      var_local_marker_start = var_local_tvt_shaper_index_display - 1;
      var_local_marker_end   = var_local_tvt_shaper_index_display;
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_set_tvt_shaper_offset_display
{
   DEFINITION
   {
      var_local_tvt_shaper_value_display = TVT_SHAPER_ARRAY[var_local_tvt_shaper_index_display - 1] * TVT_SHAPER_STEP_SIZE;
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_get_tvt_shaper_offset_display
{
   DEFINITION
   {
      int i;
      int index_start;
      int index_end;

      // make sure value is valid
      if (var_local_tvt_shaper_value_display > (TVT_SHAPER_MAXIMUM_VALUE * TVT_SHAPER_STEP_SIZE))
      {
         var_local_tvt_shaper_value_display = (TVT_SHAPER_MAXIMUM_VALUE * TVT_SHAPER_STEP_SIZE);
      }
      if (var_local_tvt_shaper_value_display < (TVT_SHAPER_MINIMUM_VALUE * TVT_SHAPER_STEP_SIZE))
      {
         var_local_tvt_shaper_value_display = (TVT_SHAPER_MINIMUM_VALUE * TVT_SHAPER_STEP_SIZE);
      }
      TVT_SHAPER_ARRAY[var_local_tvt_shaper_index_display - 1] = var_local_tvt_shaper_value_display / TVT_SHAPER_STEP_SIZE;

      // Transfer new value into TVT
      index_start = ((var_local_tvt_shaper_index_display - 1) * var_local_tvt_shaper_step_size) + var_local_tvt_shaper_zero_offset + 0.5;
      index_end   = ((var_local_tvt_shaper_index_display    ) * var_local_tvt_shaper_step_size) + var_local_tvt_shaper_zero_offset + 0.5;

      for (i = index_start; i < index_end; i++)
         {
         if ((i >= 0) && (i < NUMBER_TVT_POINTS))
            {
            array_local_tvt_point[i] = tvt_point[i] + var_local_tvt_shaper_value_display;
            }
         }
      
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_index_plus
{
   LABEL      [point_label] + " +";
   HELP       [point_help];
   DEFINITION
   {
      var_local_tvt_shaper_index_display++;
      method_get_tvt_shaper_index_display();
      method_set_tvt_shaper_offset_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_index_minus
{
   LABEL      [point_label] + " -";
   HELP       [point_help];
   DEFINITION
   {
      var_local_tvt_shaper_index_display--;
      method_get_tvt_shaper_index_display();
      method_set_tvt_shaper_offset_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_value_increase
{
   LABEL      [offset_label] + " +";
   HELP       [offset_help];
   DEFINITION
   {
      var_local_tvt_shaper_value_display++;
      method_get_tvt_shaper_offset_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_value_decrease
{
   LABEL       [offset_label] + " -";
   HELP        [offset_help];
   DEFINITION
   {
      var_local_tvt_shaper_value_display--;
      method_get_tvt_shaper_offset_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_measure
{
  DEFINITION
  {
      int tvt_shaper_mode;
      float profile_increment;
      int next_increment_index;
      int current_index;
      int tvt_point_index;

      var_local_tvt_shaper_profile_taken = 0;

      CMD_READ_TVT_SHAPER_MODE;
      tvt_shaper_mode = TVT_SHAPER_MODE;
      TVT_SHAPER_MODE = TVT_SHAPER_MODE_OFF;
      CMD_WRITE_TVT_SHAPER_MODE;

      // Delay increased from 2 seconds to 10 seconds to wait for a valid profile when rate is slow
      DELAY_TIME(10);

      METHOD_TVT_SHAPER_READ_PROFILE_DATA();

      TVT_SHAPER_MODE = tvt_shaper_mode;
      CMD_WRITE_TVT_SHAPER_MODE;

      // Turn on the mode because points may base validity on mode
      TVT_SHAPER_MODE = TVT_SHAPER_MODE_ON;

      // Compute echo profile values
      profile_increment                          = (par_x_axis_end - par_x_axis_start) / NUMBER_ECHO_POINTS;
      var_local_tvt_shaper_x_axis_echo_increment = NUMBER_TVT_SHAPER_POINTS / (par_x_axis_end / profile_increment);
      var_local_tvt_shaper_x_axis_echo_start     = (par_x_axis_start / profile_increment) * var_local_tvt_shaper_x_axis_echo_increment;

      // Compute tvt values
      profile_increment                  = (par_x_axis_end - par_x_axis_start) / NUMBER_TVT_POINTS;
      var_local_tvt_shaper_tvt_increment = NUMBER_TVT_SHAPER_POINTS / (par_x_axis_end / profile_increment);
      var_local_tvt_shaper_tvt_start     = (par_x_axis_start / profile_increment) * var_local_tvt_shaper_tvt_increment;

      var_local_tvt_shaper_zero_offset = - var_local_tvt_shaper_tvt_start / var_local_tvt_shaper_tvt_increment;
      var_local_tvt_shaper_step_size = (NUMBER_TVT_POINTS - var_local_tvt_shaper_zero_offset) / NUMBER_TVT_SHAPER_POINTS;

      // update local copy of tvt
      tvt_point_index = -1;
      next_increment_index = var_local_tvt_shaper_zero_offset;
      for (current_index = 0; current_index < NUMBER_TVT_POINTS; current_index++)
         {
         if (current_index >= next_increment_index)
            {
            tvt_point_index++;
            next_increment_index = ((tvt_point_index + 1) * var_local_tvt_shaper_step_size) + var_local_tvt_shaper_zero_offset;
            }

         if ((tvt_point_index < 0) || (tvt_point_index >= NUMBER_TVT_SHAPER_POINTS))
            {
            array_local_tvt_point[current_index] = tvt_point[current_index];
            }
         else
            {
            array_local_tvt_point[current_index] = tvt_point[current_index] + TVT_SHAPER_ARRAY[tvt_point_index] * TVT_SHAPER_STEP_SIZE;
            }
         }

      var_local_tvt_shaper_profile_taken = 1;
  }
}

/*-----------------------------------------------------------------------------
   Description: This is the method that transfers the shaper to the device
                It writes both the mode and the points
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_transfer_to_device
{
   LABEL [transfer_to_device_button_label];
   HELP  [transfer_to_device_button_help];
   DEFINITION
   {
      int point;

      //TVT_SHAPER_MODE = TVT_SHAPER_MODE_ON;
      CMD_WRITE_TVT_SHAPER_MODE;
      CMD_WRITE_TVT_SHAPER_POINTS;
      PUT_MESSAGE([tvt_shaper_successful_label]);
      method_backup_tvt_shaper();
   }
}

/*-----------------------------------------------------------------------------
   Description: This is the method that transfers the shaper from the device
                It reads both the mode and the points
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_update_from_device
{
   LABEL [method_measure_echo_profile_label];
   HELP  [method_measure_echo_profile_help];
   DEFINITION
   {
      CMD_READ_TVT_SHAPER_MODE;
      CMD_READ_TVT_SHAPER_POINTS;
      method_tvt_shaper_measure();
   }
}

/*-----------------------------------------------------------------------------
                                                         For Internal Use Only.
      Copyright (C) Siemens-Milltronics Process Instruments Inc.
      All Rights Reserved.

End of file -------------------------------------------------------------------*/
